--***********************************************************************
--  COPYRIGHT     :  GMV S.A., TRES CANTOS, MADRID
--  PROJECT       :  INTEGRAL DPE CSSW
--  FILE NAME     :  cssw_if_types_.ada
--  TYPE          :  PACKAGE SPECIFICATION
--  UNIT NAME     :  CSSW_IF_TYPES
--
--  CI.-NO.       :  I323220
--
--  COMPILER      :  TLDADA v5.7.5
--  PROCESSOR     :  1750A
--
--  FUNCTION      :  This package contains definitions necessary
--                :  to use the CSSW procedural interface (package
--                :  DPE_CSSW) from the IASW.
--                :
--                :  Such definitions include visible exceptions and
--                :  types, as well as configuration constants used
--                :  for provided types definitions.
--                :
--                :  All other declarations which are only relevant
--                :  to the internal CSSW features have been placed
--                :  in other packages along the HOOD design 
--                :  hierarchy.
--                :
--                :  Please note that the CSSW_COMMON_TYPES package
--                :  contains useful type and subtype definitions
--                :  commonly used.
--***********************************************************************
--
--  SCCS VERSION NUMBER & DATE :  2.15 03/30/99
--
--***********************************************************************

with CSSW_COMMON_TYPES;
with SYSTEM;

package CSSW_IF_TYPES is

   -------------------------------------------------------------------
   -- Task Management
   -------------------------------------------------------------------

   K_MAX_NUM_IASW_TASKS     : constant POSITIVE := 10;  --GJ 20;
   -- The maximum number of tasks that the IASW can create.
   -- This constant is used to define the upper limit of the 
   -- type T_IASW_TASK_ID.

   K_MAX_NUM_TASKS_PER_HW_EVENT : constant POSITIVE := 7;  --GJ 2;
   -- The maximum number of tasks that can be connected concurrently
   -- to the same HW event.

   EX_HW_EVENT_CONNECTION_OVERFLOW : exception;
   -- An attempt was made to connect more tasks to the same HW event
   -- than specified by K_MAX_NUM_TASKS_PER_HW_EVENT.

   EX_ALREADY_CONNECTED_TO_HW_EVENT : exception;
   -- An attempt was made to connect a task to a HW event to which it
   -- is already connected.

   -------------------------------------------------------------------
   -- Cyclic Task Management
   -------------------------------------------------------------------

   type T_RTC_FREQUENCY is
                   (F125_mHz,
                    F250_mHz,
                    F500_mHz,
                    F001_Hz,
                    F002_Hz,
                    F004_Hz,
                    F008_Hz,
                    F016_Hz,
                    F032_Hz,
                    F064_Hz,
                    F128_Hz,
                    F256_Hz);

   K_CONVERT_FREQ_TO_TICKS_PER_SEC : constant array (T_RTC_FREQUENCY) 
                                              of FLOAT := 
                   (F125_mHz => 0.125,
                    F250_mHz => 0.25,
                    F500_mHz => 0.5,
                    F001_Hz  => 1.0,
                    F002_Hz  => 2.0,
                    F004_Hz  => 4.0,
                    F008_Hz  => 8.0,
                    F016_Hz  => 16.0,
                    F032_Hz  => 32.0,
                    F064_Hz  => 64.0,
                    F128_Hz  => 128.0,
                    F256_Hz  => 256.0);

   K_RTC_FREQUENCY : constant T_RTC_FREQUENCY := F008_Hz;
   -- Default RTC frequency = 8Hz = minor cycle (period) of 125msecs.

   K_MINOR_CYCLE_IN_SECS : constant FLOAT := 0.125;
   -- The length of the minor cycle, expressed in units of seconds.
   -- 125msecs is the default. 

   K_MAX_PROCESS_PERIODS : constant POSITIVE := 10;
   -- The maximum number of periodic processes that may be cycling 
   -- concurrently.

   -------------------------------------------------------------------
   -- Telecommand and Telemetry
   -------------------------------------------------------------------

   K_IASW_ICB_TC_SUB_BUFFER_CAPACITY : constant := 8;
   -- The capacity, in terms of number of TCSPs, of the ICB Sub-Buffer
   -- used to store TCSP addressed to the IASW.

   K_ICB_SCIENCE_SUB_BUFFER_CAPACITY : constant := 20;  --GJ 50;
   -- The capacity, in terms of number of TMSPs, of the ICB Sub-Buffer
   -- used to store Science Packets.

   K_MAX_SIZE_OF_CONFIGURABLE_ICB_SUB_BUFFERS : constant := 5400;  --GJ 11992;
   -- The maximum amount of memory, in words, that may be occupied by
   -- the aggregate of the user-configurable ICB Sub-buffers, i.e. the
   -- IASW TC SB (size=124*K_IASW_ICB_TC_SUB_BUFFER_CAPACITY)
   -- and Science TM SB (size=220*K_ICB_SCIENCE_SUB_BUFFER_CAPACITY).
   -- If this size is exceeded, the CSSW initialisation procedure detects
   -- the error, generates an On-Event message, and prevents the sub-buffers
   -- from being used, by raising the "buffer-full" exception whenever an
   -- attempt is made to write to either of them.

--GJ begin
   K_JEMX_1 : constant INTEGER := 1;
   K_JEMX_2 : constant INTEGER := 2;

   K_JEMX : constant INTEGER := K_JEMX_1;
--GJ end

   K_CSSW_APID : constant INTEGER := 1536;
      -- 1024; -- SPI Nominal
      -- 1152; -- SPI Redundant
      -- 1280; -- IBIS Nominal
      -- 1408; -- IBIS Redundant
      -- 1536; -- JEM-X X1
      -- 1664; -- JEM-X X2
      -- 1792; -- OMC
   -- This constant defines the APID allocated to the CSSW for each
   -- particular DPE, i.e. the value shall be changed to the one
   -- reserved from DPE to DPE.

   K_IASW_LOW_APID  : constant INTEGER := K_CSSW_APID+1; 
   K_IASW_HIGH_APID : constant INTEGER := K_CSSW_APID+127;
   -- These constants define the range of APIDs allocated to the
   -- IASW for each particular DPE, i.e. the values shall be changed
   -- to those reserved from DPE to DPE.
   -- N.B. The example values 1025,1151 define the range of APIDs for
   -- the IASW of the nominal DPE for the SPI instrument.

   K_IASW_HK_DATA_LEN : constant POSITIVE := 145;
   -- The length, in 16-bit words, of the part assigned to the IASW 
   -- in the Packet Data Field of the HK TM SP.

   -------------------------------------------------------------------
   -- Mailboxes
   -------------------------------------------------------------------

   K_MAX_NUM_OF_IASW_MBOX  : constant POSITIVE := 4;  --GJ 20;
   -- The maximum number of mailboxes that the IASW can create.

   K_MAX_IASW_MBOX_CAPACITY     : constant POSITIVE := 5;
   -- The maximum capacity, in terms of number of messages, of a 
   -- mailbox created by the IASW. 

   K_MAX_TOTAL_IASW_MESSAGES    : constant POSITIVE := 20;  --GJ 100;
   -- The maximum number of messages that can be stored in IASW-created
   -- mailboxes globally in the mailbox system (in Astres) concurrently. 
   -- It does not make sense to set the value higher than
   -- (K_MAX_NUM_OF_IASW_MBOX*K_MAX_IASW_MBOX_CAPACITY),
   -- but it could be set lower, since it is unlikely that all mailboxes
   -- will be dimensioned to the same (maximum) size and also concurrently
   -- full. Note that ASTRES requires 5 words of data per message, 
   -- so it is advisable to avoid unecessarily over-dimensioning
   -- the mailbox system.

   K_MAX_MBOX_MSG_PRIORITY : constant POSITIVE := 10;

   --=================================================================
   -- Interface types and exceptions 
   --=================================================================

   -------------------------------------------------------------------
   -- General
   -------------------------------------------------------------------

   subtype T_ADDRESS_STATE is CSSW_COMMON_TYPES.T_UNSIGNED_4;
   -- Address states possible values are from 0 to 15.

   subtype T_LOGICAL_ADDRESS is SYSTEM.ADDRESS;
   -- A 64 Kwords range address

   type T_ADDRESS is
      record
         STATE   : T_ADDRESS_STATE;
         LOGICAL : T_LOGICAL_ADDRESS;
      end record;
   -- A fully specified address, covering the entire 1Mword of logical
   -- memory per operand or instruction space. 

   subtype T_PHYSICAL_ADDRESS is LONG_INTEGER range 0..(2**20)-1;
   -- A 1Mwords range address

   subtype T_RTC_TICKS is LONG_INTEGER range 0..LONG_INTEGER'last;

   -------------------------------------------------------------------
   -- Critical Regions 
   -------------------------------------------------------------------

   K_MAX_CRIT_REG_NESTING : constant POSITIVE := 10;
   -- The maximum permitted level of nesting of critical regions.

   EX_CRITICAL_REGION_NESTING_EXCEEDED : exception;
   -- An attempt has been made to enter more nested critical regions 
   -- than the maximum permitted.

   EX_NOT_IN_CRITICAL_REGION : exception;
   -- An attempt was made to leave a critical region when the calling 
   -- process was not in a critical region.

   EX_WAIT_IN_CRITICAL_REGION : exception;
   -- An attempt was made to put a process into waiting state while it
   -- was executing in a critical region.

   -------------------------------------------------------------------
   -- Mailboxes
   -------------------------------------------------------------------

   EX_MBOX_NOT_INITIALIZED : exception;
   -- The mailbox specified in the call has not been initialized, and 
   -- therefore is not recognized by the CSSW.

   EX_MBOX_FULL : exception;
   -- The specified mailbox is full.

   EX_MAILBOX_AREA_EXCEEDED : exception;
   -- An attempt to create a mailbox has been refused because it would
   -- have exceeded the Mailbox_Area_Max_Size.

   subtype T_IASW_MBOX_ID is INTEGER range 1..K_MAX_NUM_OF_IASW_MBOX;

   subtype T_IASW_MBOX_CAPACITY is INTEGER range 1..K_MAX_IASW_MBOX_CAPACITY;

   type T_MESSAGE is array (1..2) of INTEGER;

   subtype T_MSG_PRIORITY is INTEGER range 1..K_MAX_MBOX_MSG_PRIORITY;

   type T_OPT_MESSAGE (VALID : BOOLEAN := TRUE) is
      record
         case VALID is
            when TRUE =>
               MSG   : T_MESSAGE;
            when FALSE =>
               null;
          end case;
      end record;
   -- An optional message. 

   -------------------------------------------------------------------
   -- Task Management
   -------------------------------------------------------------------

   EX_TASK_NOT_INITIALIZED : exception;
   -- The Task specified in the call has not been initialized, and 
   -- therefore is not recognized by the CSSW.

   EX_TASK_ALREADY_STARTED : exception;
   -- The Task specified in the call to START_TASK was not in
   -- NON_OPERATIONAL state, i.e. it was already started.

   EX_TASK_ALREADY_STOPPED : exception;
   -- The Task specified in the call to STOP_TASK was in NON_OPERATIONAL
   -- state, i.e. it was already stopped.

   subtype T_IASW_TASK_ID is INTEGER range 1..K_MAX_NUM_IASW_TASKS;
   -- Identifiers that may be assigned to IASW tasks.

   subtype T_IASW_TASK_PRIORITY is INTEGER range 2..K_MAX_NUM_IASW_TASKS+7;
   -- Range of priorities that may be assigned to IASW tasks.

   K_CSSW_TASKS_BASE_PRIORITY : constant T_IASW_TASK_PRIORITY := 5;
   -- Used to set the priority of the lowest priority CSSW task belonging
   -- to the set of CSSW tasks whose priorities can be configured by the
   -- user. This "base priority" is configurable so that the user has the
   -- freedom to choose how many priority levels are between the priority
   -- of the CSSW IDLE_PROCESS and the set of CSSW Tasks comprised of
   -- TC_INPUT_BUFFER, CURRENT_TC and TM_CYCLE.

   type T_INITIAL_STATE is (STARTED, NOT_STARTED);
   -- Possible states for a task after initialisation, i.e. indicates
   -- if the task has to be started when initialised or not.

   type T_TASK_STATE is ( EXECUTING,
                          READY, 
                          WAITING,
                          NON_OPERATIONAL);
   -- Possible states of any task for the CSSW process scheduler.
   -- The ordering of the literals ensures that the positions, and 
   -- internal codes, are equal to those used by ASTRES to denote each
   -- of the possible states of a task.

   type T_TASK_STACK is array (INTEGER range <>) of INTEGER;
   -- A task stack.

   -------------------------------------------------------------------
   -- Cyclic Task Management
   -------------------------------------------------------------------

   EX_PERIOD_ID_UNKNOWN : exception;
   -- The periodic process key specified in the call is not currently 
   -- defined (it has either never been defined or has been deleted), 
   -- and therefore is not recognized by the CSSW.

   EX_MAX_PERIOD_IDS_EXCEEDED : exception;
   -- An attempt was made to define more cylic processes than permitted.

   subtype T_PERIOD_ID is INTEGER range 1..K_MAX_PROCESS_PERIODS;
   -- Range of possible identifiers for cyclic process periods.

   -------------------------------------------------------------------
   -- Events
   -------------------------------------------------------------------

   subtype T_EVENT_NUMBER is INTEGER range 1..16;

   type T_EVENT_LIST is array (T_EVENT_NUMBER) of BOOLEAN;
   pragma PACK (T_EVENT_LIST);
   -- Each task can have up to 16 associated events.
   -- Each set bit (i.e. 1 value) in the INTEGER word represents
   -- inclusion of the associated event (=index value) in the list.

   type T_HW_EVENT is ( -- Watchdog
                        WATCHDOG_OVERFLOW,
                        -- Real time clock
                        REAL_TIME_CLOCK,
                        -- EDAC
                        EDAC_MP_SINGLE_BIT_ERROR,
                        EDAC_NOT_MP_SINGLE_BIT_ERROR,
                        EDAC_MP_DOUBLE_BIT_ERROR,
                        EDAC_NOT_MP_DOUBLE_BIT_ERROR,
                        -- OBDH
                        EOTC,
                        EOTM,
                        UNKNOWN_INSTRUCTION_TO_USER,
                        BCP1,
                        START_ICU,
                        RESET_ICU,
                        UNKNOWN_INSTRUCTION_TO_RBI,
                        OBDH_CLOCK_IS_LOST,
                        -- On-board time
                        BCP4,
                        -- Low speed serial line
                        LSL_END_OF_TRANSMISSION,
                        LSL_END_OF_RECEPTION,
                        LSL_PARITY_ERROR,
                        LSL_OVERRUN_ERROR,
                        LSL_FRAMING_ERROR,
                        -- High speed serial line
                        HSL_END_OF_MESSAGE,
                        HSL_FIFO_FULL_ERROR,
                        -- Microprocessor
                        TIMER_A,
                        -- Analog acquisition
                        END_OF_ANALOG_ACQUISITION,
                        -- Hardware Interrupts
                        PARITY_ERROR,
                        IO_BUS_TIMEOUT,
                        MEMORY_BUS_TIMEOUT,
                        PARALLEL_IO_TRANSFER_ERROR,
                        ILLEGAL_INSTRUCTION_CODE,
                        UNIMPLEMENTED_ADDRESS,
                        UNKNOWN_INTERRUPT_01_SOURCE,
                        UNKNOWN_INTERRUPT_14_SOURCE,
                        MEMORY_PROTECTION_MP,
                        MEMORY_PROTECTION_NOT_MP,
                        EXTERNAL_ADDRESS_ERROR,
                        PRIVILEGED_INSTRUCTION );

   -------------------------------------------------------------------
   -- Interrupt Management
   -------------------------------------------------------------------

   type T_INTERRUPT_ID is 
      (INT0_REAL_TIME_CLOCK,
       INT1_MACHINE_ERROR,
       INT2_RBI_INSTRUCTION_TO_RBI,
       INT3_FLOATING_POINT_OVERFLOW,
       INT4_FIXED_POINT_OVERFLOW,
       INT5_BEX,
       INT6_FLOATING_POINT_OVERFLOW,
       INT7_TIMER_A_OVERFLOW,
       INT8_RBI_INSTRUCTION_TO_USER,
       INT9_TIMER_B_OVERFLOW,
       INT10_BCP1,
       INT11_BCP4,
       INT12_IO_BUS,
       INT13_DMA_CONTROLLER,
       INT14_IO_BUS,
       INT15_CPU_SERIAL_TEST_IF);
   -- The identities of each of the 15 microprocessor interrupts.

   type T_IRQ_MASK_BIT is (MASKED, NOT_MASKED);
   -- The meanings of the interrupt mask values, i.e. 0=masked,1=unmasked.

   type T_IRQ_MASK is array (T_INTERRUPT_ID) of T_IRQ_MASK_BIT;
   pragma PACK (T_IRQ_MASK);
   -- The mask to be applied to interrupts.

   type T_IRQ_PENDING_BIT is (NOT_PENDING, PENDING);
   -- The meanings of the pending interrupt register bit values, 
   -- i.e. 0=not pending, 1=pending.

   type T_IRQ_PENDING is array (T_INTERRUPT_ID) of T_IRQ_PENDING_BIT;
   pragma PACK (T_IRQ_PENDING);
   -- The contents of the pending interrupts register.

   type T_MIL_STD_1750A_FAULT_ID is
      (CPU_MEMORY_PROTECTION_FAULT,
       DMA_PROTECTION_FAULT,
       MEMORY_PARITY_FAULT,
       PIO_CHANNEL_PARITY_FAULT,
       DMA_CHANNEL_PARITY_FAULT,
       ILLEGAL_IO_COMMAND_FAULT,
       PIO_TRANSMISSION_FAULT,
       SPARE,
       ILLEGAL_ADDRESS_FAULT,
       ILLEGAL_INSTRUCTION_FAULT,
       PRIVILEGED_INSTRUCTION_FAULT,
       ADDRESS_STATE_FAULT,
       RESERVED,
       BUILT_IN_TEST_FAULT,
       SPARE_BITE_1,
       SPARE_BITE_2);
   -- The fault associated with each bit of the 1750 Fault Register, as
   -- defined by the MIL-STD-1750A(USAF).

   type T_MIL_STD_1750A_FT_BIT is (OKAY, FAULT);
   -- The meanings of the bit-values in the CPU Fault Register,
   -- i.e. 0=Fault occurred, 1=Fault not occurred or cleared. 

   type T_MIL_STD_1750A_FT is
      array (T_MIL_STD_1750A_FAULT_ID) of T_MIL_STD_1750A_FT_BIT;
   pragma PACK (T_MIL_STD_1750A_FT);
   -- The contents of the 1750A's Fault Register.

   -------------------------------------------------------------------
   -- Telecommand and Telemetry
   -------------------------------------------------------------------

   EX_APID_NOT_IN_USE : exception;
   -- The APID specified in a call is not in use by the IASW.

   EX_OE_SB_FULL : exception;
   -- The On-Event Sub-Buffer is full.

   EX_SCI_SB_FULL : exception;
   -- The Science Sub-buffer is full.

   subtype T_IASW_APID is 
              INTEGER range K_IASW_LOW_APID..K_IASW_HIGH_APID;
   -- The range of APIDs assigned to the IASW for this DPE.

   K_IASW_TC_LOW_APID : constant T_IASW_APID := K_IASW_LOW_APID;
   -- The lowest IASW APID reserved for the IASW TC handling process(es).

   K_IASW_TC_HIGH_APID : constant T_IASW_APID := K_IASW_TC_LOW_APID;
   -- The lowest IASW APID reserved for the IASW TC handling process(es).

   type T_ICB_SB_NAME is (TC_IASW, TC_CSSW, TM_SCI, TM_CSSW_OR, TM_OE);
   -- The logical identities of the ICB sub-buffers.

   type T_SB_STATE is
      record
         USED : NATURAL;
         FREE : NATURAL;
      end record;
   -- The status information available for each ICB sub-buffer, i.e. in 
   -- terms of the number of elements in use (occupied) and free (empty).

   type T_ICB_STATE is array (T_ICB_SB_NAME) of T_SB_STATE;
   -- The state of the ICB sub-buffers.

   --
   -- Telecommand definitions
   --

   type T_TC_SOURCE is (ON_GROUND, ON_BOARD);
   for T_TC_SOURCE use (ON_GROUND => 0,
                        ON_BOARD  => 1);
   -- The possible sources of a TC.

   subtype T_TC_SEQ_NUMBER is CSSW_COMMON_TYPES.T_NATURAL_13;
   -- The actual sequence count of a TC

   type T_TC_PACKET_HEAD_DATA is
      record
         SPARE : CSSW_COMMON_TYPES.T_UNSIGNED_2;
         SRC   : T_TC_SOURCE;
         SEQ   : T_TC_SEQ_NUMBER;
      end record;
   -- The TC packet header data which may be relevant to the IASW.
   -- TC fields with fixed values, as well as fields managed by the
   -- CSSW are excluded.

   for T_TC_PACKET_HEAD_DATA use
      record
         SPARE at 0 range 0..1;
         SRC   at 0 range 2..2;
         SEQ   at 0 range 3..15;
      end record;

   subtype T_TC_APPL_DATA_LEN is INTEGER range 0..120;
   -- The possible lengths (in words) of the Application Data Field of a TC 
   -- Source Packet.

   type T_TC_APPL_DATA is 
           array (1..T_TC_APPL_DATA_LEN'LAST) of INTEGER;
   -- A maximum length TC Application Data field.

   type T_TC_APPL_DATA_FIELD is
      record
         LEN   : T_TC_APPL_DATA_LEN;
         WORDS : T_TC_APPL_DATA;
      end record;
   -- A TC Application Data field, with an indication of the 
   -- length of the data. Note that the type is not discriminated 
   -- in order to avoid the use of heap memory.

   subtype T_TC_TYPE is CSSW_COMMON_TYPES.T_NATURAL_4;
   subtype T_TC_SUBTYPE is CSSW_COMMON_TYPES.T_NATURAL_4;
   -- The ranges of TC packet types and subtypes (4 bits each).

   type T_TC_DATA_FIELD_HEAD_DATA is
      record
         TYP : T_TC_TYPE;
         SUB : T_TC_SUBTYPE;
      end record;
   -- The data field header data which may be relevant to the IASW.
   -- Again, fields with fixed values or only relevant to CSSW are
   -- not included.

   type T_TCSP is
      record
         PCK_HEAD  : T_TC_PACKET_HEAD_DATA;
         DAT_HEAD  : T_TC_DATA_FIELD_HEAD_DATA;
         DATA      : T_TC_APPL_DATA_FIELD;
      end record;
   -- Those parts of  a TC Source Packet which are of interest 
   -- to IASW.

   type T_OPT_TCSP (VALID : BOOLEAN := TRUE) is
      record
         case VALID is
            when TRUE =>
               TC    : T_TCSP;
            when FALSE =>
               null;
         end case;
      end record;
   -- An optional TCSP.   

   --
   -- Telemetry definitions
   --

   type T_SCI_TM_PRIORITY is (LOW, MEDIUM, TOP);

   type T_IASW_HK_DATA_FIELD is
           array (1..K_IASW_HK_DATA_LEN) of INTEGER;
   -- The part reserved for IASW in the data field of each DPE 
   -- HK TMSP.

   subtype T_SOURCE_SEQ_COUNT is CSSW_COMMON_TYPES.T_NATURAL_14;
   -- The range of the TM Source Packet Source Sequence Count, which 
   -- goes into a 14-bit field of the Packet Sequence Control part of 
   -- the Source Packet Header.

   type T_SCIENCE_DATA_FIELD is array (1..217) of INTEGER; 
   -- The Packet Data Field including the Data Field Header and optional 
   -- CRC field, of a Science TM packet. Note that the IASW has complete 
   -- freedom over the content. 

   subtype T_IASW_ON_EVENT_ID is INTEGER range 127..254;
   -- The range of On-Event Message Identifiers assigned to the IASW.

   type T_ON_EVENT_MSG_CLASS is (EVENTS, 
                                 EXCEPTN, 
                                 MAJOR_ANOMALY,
                                 FAILED_TC_REPORT);
   -- The classes of On-Event message that the IASW is able to generate.

   type T_IASW_ON_EVENT_MSG_ID is
      record
         ID    : T_IASW_ON_EVENT_ID;
         CLASS : T_ON_EVENT_MSG_CLASS;
      end record;
   -- The complete identification of an On-Event Message, consisting 
   -- of message identifier and class.

   type T_ON_EVENT_MSG_BODY is 
      record
         FIELD_1 : CSSW_COMMON_TYPES.T_UNSIGNED_6;
         FIELD_2 : INTEGER;
         FIELD_3 : INTEGER;
      end record;
   -- The body (38-bits) of an On-Event message.

   for T_ON_EVENT_MSG_BODY use
       record
          FIELD_1 at 0 range 0..5;
          FIELD_2 at 1 range 0..15;
          FIELD_3 at 2 range 0..15;
       end record;

   --=================================================================
   --
   --                DPE SW DRIVERS
   --
   --=================================================================

   -------------------------------------------------------------------
   -- General
   -------------------------------------------------------------------

   type T_U32_BITS is
      record
         HIGH : CSSW_COMMON_TYPES.T_UNSIGNED_16;
         LOW  : CSSW_COMMON_TYPES.T_UNSIGNED_16;
      end record;

   -------------------------------------------------------------------
   -- High Speed Serial Lines
   -------------------------------------------------------------------

   EX_HIGH_SPEED_LINE_IS_BUSY : exception;

   type T_HIGH_SPEED_LINE is ( A, B );

   type T_FIFO_STATUS is ( EMPTY, BENEATH_HALF, ABOVE_HALF, FULL );

   type T_HIGH_SPEED_LINE_STATUS is
   record
      SELECTED_LINE           : T_HIGH_SPEED_LINE;
      FIFO_STATUS             : T_FIFO_STATUS;
      REMAINING_PACKET_LENGTH : SYSTEM.UNSIGNED;
      IS_ENABLED              : BOOLEAN;
      DMA_IS_ENABLED          : BOOLEAN;
      RX_IS_IN_PROGRESS       : BOOLEAN;
   end record;

   type T_WORD_TRANSFER_INTERVAL is
     ( CLK_000,
       CLK_032,
       CLK_064,
       CLK_096,
       CLK_128,
       CLK_160,
       CLK_192,
       CLK_224,
       CLK_256,
       CLK_288,
       CLK_320,
       CLK_352,
       CLK_384,
       CLK_416,
       CLK_448,
       CLK_480 );

   -------------------------------------------------------------------
   -- Low Speed Serial Lines
   -------------------------------------------------------------------

   EX_LOW_SPEED_LINE_IS_DISABLED  : exception;

   EX_LOW_SPEED_LINE_IS_BUSY : exception;

   EX_UNAUTHORIZED_BAUD_RATE : exception;

   subtype T_LOW_SPEED_LINE_CHANNEL is
              CSSW_COMMON_TYPES.T_UNSIGNED_16 range 0..3;

   type T_RTS_LINE_LEVEL is ( LOW, HIGH );

   subtype T_TRANSMITTER_ADDRESS is
              CSSW_COMMON_TYPES.T_UNSIGNED_8;

   type T_BAUD_RATE is
     ( KBPS_64,
       KBPS_32,
       KBPS_16,
       KBPS_08,
       KBPS_04,
       KBPS_02 );

   type T_CURRENT_TRANSACTION_STATUS is
          ( IDLE,
            TX_ON_COURSE,
            RX_ON_COURSE,
            PARITY_ERROR,
            OVERRUN_ERROR,
            FRAMING_ERROR);

   type T_LOW_SPEED_LINE_STATUS is 
      record
         IS_ENABLED          : BOOLEAN;
         BAUD_RATE           : T_BAUD_RATE;
         CHANNEL             : T_LOW_SPEED_LINE_CHANNEL;
         TRANSMITTER_ADDRESS : T_TRANSMITTER_ADDRESS;
         TRANSACTION         : T_CURRENT_TRANSACTION_STATUS;
      end record;

   -------------------------------------------------------------------
   -- Analog Channels
   -------------------------------------------------------------------

   EX_BILEVEL_USED_AS_ANALOG_CHANNEL  : exception;

   EX_ANALOG_CHANNEL_USED_AS_BILEVEL  : exception;

   EX_ANALOG_ACQUISITION_ERROR        : exception;

   EX_ANALOG_ACQUISITION_IS_BUSY      : exception;

   type T_ANALOG_CHANNEL is
        ( DC_5V_SECONDARY_VOLTAGE,
          DC_RAM_5V_SECONDARY_VOLTAGE,
          DC_15V_RELAY_SECONDARY_VOLTAGE,
          DC_15V_SECONDARY_VOLTAGE,
          DC_MINUS_15V_SECONDARY_VOLTAGE,
          DPE_TEMPERATURE,

          ANALOG_ACQUISITION_0,
          ANALOG_ACQUISITION_1,
          ANALOG_ACQUISITION_2,
          ANALOG_ACQUISITION_3,
          ANALOG_ACQUISITION_4,
          ANALOG_ACQUISITION_5,
          ANALOG_ACQUISITION_6,
          ANALOG_ACQUISITION_7,
          ANALOG_ACQUISITION_8,
          ANALOG_ACQUISITION_9,
          ANALOG_ACQUISITION_10,
          ANALOG_ACQUISITION_11,
          ANALOG_ACQUISITION_12,
          ANALOG_ACQUISITION_13,
          ANALOG_ACQUISITION_14,
          ANALOG_ACQUISITION_15,

          PT500_THERMISTOR_0,
          PT500_THERMISTOR_1,
          PT500_THERMISTOR_2,
          PT500_THERMISTOR_3,
          PT500_THERMISTOR_4,
          PT500_THERMISTOR_5,
          PT500_THERMISTOR_6,
          PT500_THERMISTOR_7,
          PT500_THERMISTOR_8,
          PT500_THERMISTOR_9,

          YSI_THERMISTOR_0,
          YSI_THERMISTOR_1,
          YSI_THERMISTOR_2,
          YSI_THERMISTOR_3,
          YSI_THERMISTOR_4,
          YSI_THERMISTOR_5,

          -- First MUX stage reference channels

          REFERENCE_CHANNEL_1,
          REFERENCE_CHANNEL_2,
          REFERENCE_CHANNEL_3,
          REFERENCE_CHANNEL_4,

          -- references to digital ground

          REFERENCE_CHANNEL_5,
          REFERENCE_CHANNEL_6,
          REFERENCE_CHANNEL_7,
          REFERENCE_CHANNEL_8,
          REFERENCE_CHANNEL_9,
          REFERENCE_CHANNEL_10,
          REFERENCE_CHANNEL_11,
          REFERENCE_CHANNEL_12,

          -- references for DC/DC Monitoring

          REFERENCE_CHANNEL_13,
          REFERENCE_CHANNEL_14,
          REFERENCE_CHANNEL_15,
          REFERENCE_CHANNEL_16,
          REFERENCE_CHANNEL_17,
          REFERENCE_CHANNEL_18,
          REFERENCE_CHANNEL_19,
          REFERENCE_CHANNEL_20,
          REFERENCE_CHANNEL_21,

          -- references for analog acquisition

          REFERENCE_CHANNEL_22,

          -- references for PT500 Thermistors

          REFERENCE_CHANNEL_23,
          REFERENCE_CHANNEL_24,
          REFERENCE_CHANNEL_25,
          REFERENCE_CHANNEL_26,
          REFERENCE_CHANNEL_27,

          -- references for YSI Thermistors

          REFERENCE_CHANNEL_28,
          REFERENCE_CHANNEL_29,
          REFERENCE_CHANNEL_30,
          REFERENCE_CHANNEL_31,
          REFERENCE_CHANNEL_32,
          REFERENCE_CHANNEL_33,
          REFERENCE_CHANNEL_34,
          REFERENCE_CHANNEL_35,
          REFERENCE_CHANNEL_36,
          REFERENCE_CHANNEL_37 );

   subtype T_ANALOG_VALUE is CSSW_COMMON_TYPES.T_UNSIGNED_8;

   subtype T_BILEVEL_VALUE is INTEGER range 0..1;

   type T_ANALOG_ACQUISITION_STATUS is
     record
          CONVERSION_IS_OVER : BOOLEAN;
     end record;

   -------------------------------------------------------------------
   -- Relays
   -------------------------------------------------------------------

   EX_THE_RELAY_MODULE_IS_BUSY      : exception;

   EX_A_PULSE_IS_STILL_ACTIVE       : exception;

   subtype T_RELAY_ID is INTEGER range 0..7;

   type T_CONTACT_IS_CLOSED is array (T_RELAY_ID) of BOOLEAN;


   -------------------------------------------------------------------
   -- On-Board Time
   -------------------------------------------------------------------

   type T_OBT is
      record
         HIGH : CSSW_COMMON_TYPES.T_UNSIGNED_16;
         MID  : CSSW_COMMON_TYPES.T_UNSIGNED_16;
         LOW  : CSSW_COMMON_TYPES.T_UNSIGNED_16;
      end record;

   -------------------------------------------------------------------
   -- Microprocessor
   -------------------------------------------------------------------

   type T_1750_TIMER_ID is ( A );
   -- Timer B is not included, because it is reserved for internal use by the 
   -- CSSW (the Timer B interrupt is used to drive the ASTRES-1750 scheduler).

   type T_1750_TIMER_CONFIGURATION is
          record
               TIMER_A_PERIOD : CSSW_COMMON_TYPES.T_UNSIGNED_16;
          end record;
 
   type T_1750_TIMER_STATUS is
          record
               REMAINING_TIME_FOR_A: CSSW_COMMON_TYPES.T_UNSIGNED_16;
               IS_A_PENDING:         BOOLEAN;
               IS_A_UNMASKED:        BOOLEAN;
          end record;

   type T_RESET_SOURCE is
          ( POWER_UP, 
            POWER_SAVING, 
            INST_RBI_RESET_00, -- Reset with parameter 0, i.e. Cold Reset
            WATCHDOG_RESET,
            EDAC_DOUBLE_FAILURE );

   -------------------------------------------------------------------
   -- Watchdog
   -------------------------------------------------------------------

   K_WDOG_EXPIRY_TIME : constant POSITIVE := 2560;  --GJ 48;
   -- The watchdog expiry time in units of (1/256) secs.
   -- The default value 48 is equal to 187.5msecs, i.e. 1.5 RTCs
   -- (so that the CPU could be hogged for up to one and a half 
   -- RTCs without causing an overrun. Note that if a temporary
   -- overrun of one RTC occurs, this is notified via the CPU Margin
   -- housekeeping information in the DPE HK TM SP.

   K_CPU_RESET_ON_WDOG_EXPIRY : constant BOOLEAN := TRUE;
   -- Indicates whether the watchdog is to reset the CPU when it expires. 

end CSSW_IF_TYPES;

