----------------------------------------------------------------------------
-- Module Name:        IASW_EVSORT - package body
-- Project:            INTEGRAL, JEM-X, DPE
-- Author:             Adam Buraczewski
-- Creation Date:      16.III.1998
-- Change History:
--    (Version/Date/Author/Description)
-- Function:
--    (Say what the component does)
--    Sorts events according to XY positions.
-- Interfaces:
--    (Define the inputs and outputs)
-- Dependencies:
--    (Describe the preconditions for using this component)
-- Processing:
--    (Summarise the processing using pseudo-code or PDL)
--    The whole sorting process goes in the following steps:
--    1) Necessary ammount of events must be stored in the big buffer.
--    2) The list of sorting keys must be prepared with use of procedures:
--       CLEAR_SORTING_KEYS_LIST and ADD_SORTING_KEY.  The sorting key
--       contains XY of the event and its position in the big buffer.
--    3) Do the sorting with use of SORT_EVENTS.  For the nominal number of
--       2400 events it takes about 1.5 second.
--    4) Peek events from the big buffer in the sorted order with use of
--       PEEK_SORTED_EVENT.
----------------------------------------------------------------------------

with IASW_COMMON;
with DPE_CSSW;

package body IASW_EVSORT is


   ------------------------------------------------------------------------
   -- QSORT_EVENTS
   --
   -- Sorts events using quick sort algorithm.

   procedure QSORT_EVENTS is
      -- Stack used by QSort algorithm.  It should be greater or equal to
      -- log2(MAX_AMOUNT_OF_EVENTS_TO_SORT).
      MAX_QSTACK_SIZE : constant NATURAL := 13;
      type T_QSTACK is record
         L, R : IASW_BUFFER.T_BUFFER_INDEX;
      end record;
      QSTACK : array (1 .. MAX_QSTACK_SIZE) of T_QSTACK;

      I, J, L, R : IASW_BUFFER.T_BUFFER_INDEX;
      X, W : T_SORT_DATA;
      S : NATURAL range 0 .. MAX_QSTACK_SIZE;
   begin
      S := 1;
      QSTACK(1).L := 1;
      QSTACK(1).R := AMOUNT_OF_SORTED_EVENTS;

      loop

         L := QSTACK(S).L;
         R := QSTACK(S).R;
         S := S - 1;

         loop

            I := L;
            J := R;
            X := SORTED_EVENTS((L + R) / 2 - 1);

            loop

               while SORTED_EVENTS(I - 1).KEY < X.KEY loop
                  I := I + 1;
               end loop;

               while X.KEY < SORTED_EVENTS(J - 1).KEY loop
                  J := J - 1;
               end loop;

               if I <= J then
                  W := SORTED_EVENTS(I - 1);
                  SORTED_EVENTS(I - 1) := SORTED_EVENTS(J - 1);
                  SORTED_EVENTS(J - 1) := W;
                  I := I + 1;
                  J := J - 1;
               end if;

               exit when I > J;
            end loop;

            if J - L < R - I then
               if I < R then
                  S := S + 1;
                  QSTACK(S).L := I;
                  QSTACK(S).R := R;
               end if;
               R := J;
            else
               if L < J then
                  S := S + 1;
                  QSTACK(S).L := L;
                  QSTACK(S).R := J;
               end if;
               L := I;
            end if;

            exit when L >= R;
         end loop;

         exit when S = 0;
      end loop;
   end QSORT_EVENTS;

   ------------------------------------------------------------------------
   -- SORT_EVENTS
   --
   -- Sorts events using currently selected method.

   procedure SORT_EVENTS is
   begin
      -- Abort when there is no events suitable for sorting.
      if AMOUNT_OF_SORTED_EVENTS < 2 then
         return;
      end if;

      -- Perform sorting.
      QSORT_EVENTS;
   end SORT_EVENTS;

   ------------------------------------------------------------------------
   -- PEEK_SORTED_EVENT
   --
   -- Similar to IASW_BUFFER.PEEK_EVENT, but returns sorted events.  Using
   -- IASW_BUFFER.PEEK_EVENT after sorting has no effect.
   --
   -- Exceptions thrown: EX_INDEX_OUT_OF_RANGE.

   procedure PEEK_SORTED_EVENT (
      INDEX : in IASW_BUFFER.T_BUFFER_INDEX;
      EVENT : in out IASW_COMMON.T_EVENT
   ) is
   begin
      if INDEX >= AMOUNT_OF_SORTED_EVENTS then
         raise IASW_BUFFER.EX_INDEX_OUT_OF_RANGE;
      end if;

      IASW_BUFFER.PEEK_EVENT (
         INDEX => SORTED_EVENTS(INDEX).BUFFER_POS,
         EVENT => EVENT);
   end PEEK_SORTED_EVENT;

   ------------------------------------------------------------------------
   -- GET_AMOUNT_OF_SORTED_EVENTS
   --
   -- Returns the amount of events that have been sorted.

   function GET_AMOUNT_OF_SORTED_EVENTS return IASW_BUFFER.T_BUFFER_INDEX is
   begin
      return AMOUNT_OF_SORTED_EVENTS;
   end GET_AMOUNT_OF_SORTED_EVENTS;


   procedure CLEAR_SORTING_KEYS_LIST is
   begin
      AMOUNT_OF_SORTED_EVENTS := 0;
   end CLEAR_SORTING_KEYS_LIST;

   procedure ADD_SORTING_KEY (
      BUFFER_POS : in IASW_BUFFER.T_BUFFER_INDEX;
      KEY        : in T_POSITION_KEY
   ) is
   begin
      if AMOUNT_OF_SORTED_EVENTS >= MAX_AMOUNT_OF_EVENTS_TO_SORT then
         -- tu mogloby byc zgloszenie wyjatku, ale chyba nie warto
         return;
      end if;

      SORTED_EVENTS(AMOUNT_OF_SORTED_EVENTS).BUFFER_POS := BUFFER_POS;
      SORTED_EVENTS(AMOUNT_OF_SORTED_EVENTS).KEY        := KEY;

      IASW_COMMON.INC(AMOUNT_OF_SORTED_EVENTS);
   end ADD_SORTING_KEY;

end IASW_EVSORT;
