----------------------------------------------------------------------------
-- Module Name:        IASW_GF - package body
-- Project:            INTEGRAL, JEM-X, DPE
-- Author:             Grzegorz Juchnikowski
-- Creation Date:      16.III.1998
-- Change History:
--    (Version/Date/Author/Description)
-- Function:
--    (Say what the component does)
-- Interfaces:
--    (Define the inputs and outputs)
-- Dependencies:
--    (Describe the preconditions for using this component)
-- Processing:
--    (Summarise the processing using pseudo-code or PDL)
----------------------------------------------------------------------------

with IASW_COMMON;
with DPE_CSSW;
with IASW_HSL;
with IASW_DFEE;

package body IASW_GF is

   ------------------------------------------------------------------------
   -- SET_EVENTS_PER_TM_BLOCK
   --
   -- This will be called at the start of an imaging format
   -- WARNING!  Output for Spectrum format is not realistic.  The real
   -- number is much larger.  The number 3000 is sufficiently small not to
   -- generate problems when multiplied by TM_BLOCKS_PER_BCP1.

   procedure SET_EVENTS_PER_TM_BLOCK(
     SCF : in IASW_MODES.T_SCIENCE_FORMAT;
     EVENT_SIZE : in IASW_COMMON.T_EVENT_SIZE
   ) is
   begin
      case SCF is
	 when IASW_MODES.K_SCF_FULL       => EVENTS_PER_TM_BLOCK := 100;
	 when IASW_MODES.K_SCF_RESTRICTED => EVENTS_PER_TM_BLOCK := 380;
	 when IASW_MODES.K_SCF_SPECT_TIM  => EVENTS_PER_TM_BLOCK := 200;
	 when IASW_MODES.K_SCF_TIMING     => EVENTS_PER_TM_BLOCK := 400;
	 when IASW_MODES.K_SCF_SPECTRUM   => EVENTS_PER_TM_BLOCK := 3000;
	 when IASW_MODES.K_SCF_CALIBRATION=> EVENTS_PER_TM_BLOCK := 6;
	 when IASW_MODES.K_SCF_DIAGNOSTIC => EVENTS_PER_TM_BLOCK := 5;
	 when IASW_MODES.K_SCF_TEST       => EVENTS_PER_TM_BLOCK :=
            IASW_COMMON.CVT(CIT.T_SCIENCE_DATA_FIELD'Last-1) / EVENT_SIZE;
	 when others                      => EVENTS_PER_TM_BLOCK := 0;
      end case;
      ASS_TM_RATE := TM_BLOCKS_PER_BCP1 * EVENTS_PER_TM_BLOCK;
   end SET_EVENTS_PER_TM_BLOCK;


   ------------------------------------------------------------------------
   -- SET_TM_BLOCKS_PER_BCP1
   --
   -- This will be called at the reception of BCPKT

   procedure SET_TM_BLOCKS_PER_BCP1 (BLNO : in CCT.T_UNSIGNED_16) is
   begin
      TM_BLOCKS_PER_BCP1 := BLNO;
      ASS_TM_RATE := TM_BLOCKS_PER_BCP1 * EVENTS_PER_TM_BLOCK;
   end SET_TM_BLOCKS_PER_BCP1;


   ------------------------------------------------------------------------
   -- SET_GREY_FILTER_ACCORDING_TO_EVENTS_RATES
   --
   -- local

   procedure SET_GREY_FILTER_ACCORDING_TO_EVENTS_RATES(
      GF : in out T_GREY_FILTER_INDICATOR
   ) is
      INPUT_RATE, OUTPUT_RATE, GFW : CCT.T_UNSIGNED_16;
   begin
      -- Take INPUT_RATE as the events-rate observed on the HSL input.
      -- In a stable state INPUT_RATE = sky-events-rate * grey-filter.
      INPUT_RATE := IASW_HSL.GET_EVENTS_RATE;

      -- Take OUTPUT_RATE as ASS_TM_RATE - calculated from the BCPKT data.
      OUTPUT_RATE := ASS_TM_RATE;

      if INPUT_RATE < 32 then
	 -- this is to eliminate / 0 in the main equation.
	 GF := GREY_FILTER_100;
      elsif GF = 0 then
	 -- in this case the main equation would return 0 forever.
	 GF := GF + 1;
      else
	 GFW := (GF * (OUTPUT_RATE / 32)) / (INPUT_RATE / 32);
	 -- this comes from the following:
	 -- 1) old_input_rate = detector_events_rate * old_grey_filter;
	 -- 2) wanted_input_rate = output_rate;
	 -- 3) new_grey_filter = wanted_input_rate / detector_events_rate;
	 -- / 32 is to eliminate possibility of overflow GF*OUTPUT_RATE.
	 if GFW in T_GREY_FILTER_INDICATOR then
	    GF := GFW;
	 else
	    GF := GREY_FILTER_100;
	 end if;
      end if;
   end SET_GREY_FILTER_ACCORDING_TO_EVENTS_RATES;


   procedure INIT_GF is
   -- called in IASW_HSL.LOCAL_START_DATA_TAKING
   begin
      PREVIOUS_BUFFER_LEVEL := 0;
   end INIT_GF;


   procedure SET_GF is
   -- called in IASW_STATE.CHANGE_STATE just after Goto Data Taking
   begin
      if AUTOMATIC_GREY_FILTER_SELECTION then
        GREY_FILTER_WANTED := GREY_FILTER_100;
      else
        null;  -- GREY_FILTER_WANTED unchanged when not automatic mode
      end if;
      IASW_DFEE.SET_GREY_FILTER(GREY_FILTER_WANTED);
   end SET_GF;


   ------------------------------------------------------------------------
   -- ANALYSE_GREY_FILTER
   --
   -- Makes an analisys of how main buffer is filled with scientific data
   -- and recalculates GREY_FILTER_WANTED when necessary.
   -- the procedure IASW_DFEE.SET_GREY_FILTER(); is called if needed.

   procedure ANALYSE_GREY_FILTER is
      N, Np  : IASW_BUFFER.T_BUFFER_INDEX;   -- current and previous
      LEVEL  : INTEGER range 0 .. 4;  -- where we are with buffer filling
      NEW_GF : T_GREY_FILTER_INDICATOR;
      dGF    : INTEGER;
      W      : CCT.T_UNSIGNED_16;
   begin
      if not AUTOMATIC_GREY_FILTER_SELECTION then
         return;
      end if;

      -- Determine the level of events in the buffer.
      N := IASW_BUFFER.GET_AMOUNT_OF_EVENTS;
      Np := PREVIOUS_BUFFER_LEVEL;
      PREVIOUS_BUFFER_LEVEL := N;
      NEW_GF := GREY_FILTER_WANTED;    -- preliminary assumption

      dGF := 0;

      if N > Np then     -- buffer goes up
         if (N > NORMAL_HIGH_EVENT_THRESHOLD) and (ASS_TM_RATE > 0) then
            dGF := - INTEGER(1 + (N-Np)/ASS_TM_RATE);
         end if;
      elsif N < 1000 then
         dGF := +1;
      elsif N < Np then  -- buffer goes down
         if N > EXTRA_HIGH_EVENT_THRESHOLD then
            dGF := -1;
         elsif N < NORMAL_LOW_EVENT_THRESHOLD then
            dGF := +1;
         end if;
      end if;

      if dGF > 0 then
         W := NEW_GF + IASW_COMMON.CVT(dGF);
         if W in T_GREY_FILTER_INDICATOR then
            NEW_GF := W;
         else
            NEW_GF := GREY_FILTER_100;
         end if;
      elsif dGF < 0 then
         dGF := -dGF;
         W := IASW_COMMON.CVT(dGF);
         if W <= NEW_GF then
            W := NEW_GF - W;
            if W in T_GREY_FILTER_INDICATOR then
               NEW_GF := W;
            else
               NEW_GF := GREY_FILTER_0;
            end if;
         else
            NEW_GF := GREY_FILTER_0;
         end if;
      end if;

      -- Set new GF.
      if NEW_GF /= GREY_FILTER_WANTED then
         if IASW_DFEE.GET_STATE > IASW_DFEE.K_STATE_SETUP then
            IASW_DFEE.SET_GREY_FILTER (NEW_GF);
            GREY_FILTER_WANTED := NEW_GF;
         end if;
      end if;
   end ANALYSE_GREY_FILTER;


   -- ---------------------------------------------------------------------
   -- SET_GREY_FILTER
   --
   -- Checks if parameters given in TC(5,3), TID = 4, FID = 43 are correct
   -- and then changes GF.
   -- the procedure IASW_DFEE.SET_GREY_FILTER(); is called if needed.
   --
   -- Exceptions thrown: EX_SW_PARAM_ERROR_VALUE.

   procedure SET_GREY_FILTER (
      PARAM_VAL : in IASW_COMMON.T_TC_PARAM_VAL;
      PARAM_LEN : in IASW_COMMON.T_SW_PARAM_LEN
   ) is
   begin
      if PARAM_LEN /= 1 then
	 raise IASW_COMMON.EX_SW_PARAM_ERROR_LENGTH;
      end if;

      if PARAM_VAL(1) = 16#FFFF# then
	 AUTOMATIC_GREY_FILTER_SELECTION := TRUE;
      elsif PARAM_VAL(1) in T_GREY_FILTER_INDICATOR then
	 GREY_FILTER_WANTED := PARAM_VAL(1);
	 AUTOMATIC_GREY_FILTER_SELECTION := FALSE;
         -- DFEE command should be the last operation in this procedure,
         -- because the above assignements should be done, even if
         -- the DFEE command fails.
	 IASW_DFEE.SET_GREY_FILTER (PARAM_VAL(1));
      else
	 raise IASW_COMMON.EX_SW_PARAM_ERROR_VALUE;
      end if;
   end SET_GREY_FILTER;


   -- ---------------------------------------------------------------------
   -- CALCULATE_BUFFER_THRESHOLDS
   --
   -- Calculates and sets new buffer thresholds from the values given in %
   -- (between 0 and 100).  Should be called before every start of new data
   -- mode which uses automatic grey filter selection.

   procedure CALCULATE_BUFFER_THRESHOLDS is
      UNIT : IASW_BUFFER.T_BUFFER_INDEX;
   begin
      UNIT := (IASW_BUFFER.GET_TOTAL_BUFFER_SIZE + 50) / 100;
      -- AB: Unsafe!  Overflow possible if buffer size bigger than 65485
      -- events.
      -- GJ: buffer size is 60000 and will not be more.
      EXTRA_LOW_EVENT_THRESHOLD   := EXTRA_LOW_LEVEL   * UNIT;
      NORMAL_LOW_EVENT_THRESHOLD  := NORMAL_LOW_LEVEL  * UNIT;
      NORMAL_HIGH_EVENT_THRESHOLD := NORMAL_HIGH_LEVEL * UNIT;
      EXTRA_HIGH_EVENT_THRESHOLD  := EXTRA_HIGH_LEVEL  * UNIT;
   end CALCULATE_BUFFER_THRESHOLDS;


   -- ---------------------------------------------------------------------
   -- SET_BUFFER_LEVELS
   --
   -- Checks if parameters given in TC(5,3), TID = 4, FID = 46 are correct
   -- and sets new buffer levels used later to calculate grey filter
   -- automatically.
   --
   -- Exceptions thrown: EX_SW_PARAM_ERROR_VALUE.

   procedure SET_BUFFER_LEVELS (
      PARAM_VAL : in IASW_COMMON.T_TC_PARAM_VAL;
      PARAM_LEN : in IASW_COMMON.T_SW_PARAM_LEN
   ) is
   begin
      if PARAM_LEN /= 4 then
	 raise IASW_COMMON.EX_SW_PARAM_ERROR_LENGTH;
      end if;

      if (PARAM_VAL (1) < 0) or
	 (PARAM_VAL (2) < PARAM_VAL (1)) or
	 (PARAM_VAL (3) < PARAM_VAL (2)) or
	 (PARAM_VAL (4) < PARAM_VAL (3)) or
	 (PARAM_VAL (4) > 100) then
	 raise IASW_COMMON.EX_SW_PARAM_ERROR_VALUE;
      end if;

      EXTRA_LOW_LEVEL   := PARAM_VAL(1);
      NORMAL_LOW_LEVEL  := PARAM_VAL(2);
      NORMAL_HIGH_LEVEL := PARAM_VAL(3);
      EXTRA_HIGH_LEVEL  := PARAM_VAL(4);
   end SET_BUFFER_LEVELS;


   procedure GET_BUFFER_LEVELS (
      PARAM_VAL : out IASW_COMMON.T_TM_PARAM_VAL;
      PARAM_LEN : out IASW_COMMON.T_SW_PARAM_LEN
   ) is
   begin
      PARAM_LEN := 4;
      if PARAM_VAL'length < 4 then
	 raise IASW_COMMON.EX_SW_PARAM_ERROR_LENGTH;
      end if;
      PARAM_VAL(1) := EXTRA_LOW_LEVEL;
      PARAM_VAL(2) := NORMAL_LOW_LEVEL;
      PARAM_VAL(3) := NORMAL_HIGH_LEVEL;
      PARAM_VAL(4) := EXTRA_HIGH_LEVEL;
   end GET_BUFFER_LEVELS;


   procedure GET_DPE_STATE(DS : out IASW_COMMON.T_DPE_STATE) is
   begin
      DS.GF(1) := CCT.T_UNSIGNED_8(GREY_FILTER_WANTED);
   end GET_DPE_STATE;


   procedure GET_HK_DIAG( FLAGS : out IASW_COMMON.T_16_BITS ) is
   begin
      FLAGS(10) := AUTOMATIC_GREY_FILTER_SELECTION;
   end GET_HK_DIAG;


   procedure CONSTRUCTOR is
   begin
      EVENTS_PER_TM_BLOCK := 0;
      SET_TM_BLOCKS_PER_BCP1 (8);    -- default

      AUTOMATIC_GREY_FILTER_SELECTION := TRUE;
      GREY_FILTER_WANTED      := GREY_FILTER_100;

      -- Set current buffer levels to default values.  The exact amounts
      -- of events will be calculated later, just before starting data
      -- taking.
      EXTRA_LOW_LEVEL   := DEFAULT_EXTRA_LOW_LEVEL;
      NORMAL_LOW_LEVEL  := DEFAULT_NORMAL_LOW_LEVEL;
      NORMAL_HIGH_LEVEL := DEFAULT_NORMAL_HIGH_LEVEL;
      EXTRA_HIGH_LEVEL  := DEFAULT_EXTRA_HIGH_LEVEL;
   end CONSTRUCTOR;

end IASW_GF;
