----------------------------------------------------------------------------
-- Module Name:        IASW_HSL - package body
-- Project:            INTEGRAL, JEM-X, DPE
-- Author:             Grzegorz Juchnikowski
-- Creation Date:      16.III.1998
-- Change History:
--    (Version/Date/Author/Description)
-- Function:
--    (Say what the component does)
--    Task HSL - reads events from HSL and puts them to tyhe big buffer.
-- Interfaces:
--    (Define the inputs and outputs)
-- Dependencies:
--    (Describe the preconditions for using this component)
-- Processing:
--    (Summarise the processing using pseudo-code or PDL)
----------------------------------------------------------------------------

with SYSTEM;             use SYSTEM;
with UNCHECKED_CONVERSION;
with CSSW_IF_TYPES;      use CSSW_IF_TYPES;
with CSSW_COMMON_TYPES;  use CSSW_COMMON_TYPES;
with DPE_CSSW;
with IASW_COMMON;
with IASW_BUFFER;
with IASW_MODES;
with IASW_MRTU;
with IASW_TM;
with IASW_HK;
with IASW_DFEE;
with IASW_REQ_OUT;
with IASW_GF;
with IASW_MAIN;    -- because of EX_DATA, TID_HSL

package body IASW_HSL is

   ------------------------------------------------------------------------
   -- procedures for Lost Synchro. Diagnostic Dump
   -- At each loss of synchro on HSL the procedure SET_LSD is called.
   -- It then stores fragment of HSL input buffer in a separate buffer,
   -- which next can be send in form of TM(5,4),TID=3,FID=1 on request
   -- or automatically, if so enabled with uso of TC(5,3),TID=3,FID=1.
   --

   procedure CLEAR_LSD is
   begin
      LSD.EVENT_SIZE := 0;
      LSD.HSL_BUFFER_WORDS := 0;
      LSD.HSL_BUFFER_INDEX := 0;
      LSD.LSD_OFFSET := 0;
      for I in LSD.HSL_BUFFER'range loop
         LSD.HSL_BUFFER(I) := 0;
      end loop;
   end CLEAR_LSD;

   procedure SET_LSD is
   -- called when HSL synchro. is just lost.
      N : IASW_COMMON.T_EVENT_SIZE;
   begin
      CLEAR_LSD;
      LSD.EVENT_SIZE := HSL_EVENT_SIZE;
      LSD.HSL_BUFFER_WORDS := HSL_BUFFER_WORDS;
      LSD.HSL_BUFFER_INDEX := HSL_BUFFER_INDEX;
      if HSL_BUFFER_INDEX >= HSL_EVENT_SIZE then
         LSD.LSD_OFFSET := HSL_BUFFER_INDEX - HSL_EVENT_SIZE;
      else
         LSD.LSD_OFFSET := 0;
      end if;
      N := HSL_BUFFER_WORDS - LSD.LSD_OFFSET;
      if N > LSD.HSL_BUFFER'last then
         N := LSD.HSL_BUFFER'last;
      end if;
      for I in 1..N loop
         LSD.HSL_BUFFER(I) := HSL_BUFFER(LSD.LSD_OFFSET+I-1);
      end loop;
   end SET_LSD;

   procedure SEND_LSD is
      SW_ID : constant IASW_COMMON.T_PARAM_ID := (
         TID => IASW_MAIN.TID_HSL,
         FID => 1);
      REQUEST : IASW_REQ_OUT.T_ON_RQ_TM_REQUEST;
   begin
      DPE_CSSW.ENTER_CRITICAL_REGION;
      if not IASW_REQ_OUT.REQUESTS_QUEUE_FULL then
         REQUEST.CODE := IASW_REQ_OUT.TASK_PARAMETER_REPORT;
         REQUEST.PARAM_ID := SW_ID;
         REQUEST.OEMF2 := -1;
         REQUEST.OEMF3 := -1;
         IASW_REQ_OUT.ADD_REQUEST(REQUEST);
      end if;
      DPE_CSSW.LEAVE_CRITICAL_REGION;
   end SEND_LSD;

   ------------------------------------------------------------------------

   procedure CLEAR_HSL_BUFFER is
   -- this makes the HSL input buffer empty.
   begin
      HSL_BUFFER_INDEX := 0;
      HSL_BUFFER_WORDS := 0;
   end CLEAR_HSL_BUFFER;


   procedure SWITCH_SCF is
   -- generates dummy event with the command "change format".
      DUMEV : IASW_COMMON.T_EVENT;
   begin
      DUMEV(0) := 0;
      DUMEV(1) := 0;
      DUMEV(2) := 16#FFFF#;
      DUMEV(3) := 11;
      DUMEV(4) := SCF_TAB(SCF_SELECTED) + 256*EVENT_SIZE;
      IASW_BUFFER.PUT_EVENT(DUMEV);
      IASW_GF.SET_EVENTS_PER_TM_BLOCK (SCF_TAB(SCF_SELECTED), EVENT_SIZE);
   end SWITCH_SCF;


   procedure FORMAT_SWITCHING_ANALISIS is
   -- decides if the format should be changed
   begin
      if SCF_TAB(1) = SCF_TAB(2) then
         -- primary and secondary formats are equal, there is no need
         -- for analisis.
         return;
      end if;

      case SCF_SELECTED is
   when 1 =>
         if GREY_FILTER_REPORTED < PRIMARY_MIN_GF then
            SCF_SELECTED := 2;
            SWITCH_SCF;
         end if;
   when 2 =>
         if GREY_FILTER_REPORTED > SECONDARY_MAX_GF then
            SCF_SELECTED := 1;
            SWITCH_SCF;
         end if;
   when others =>
         null;
      end case;
   end FORMAT_SWITCHING_ANALISIS;


   function HSL_EVENTS_LEFT return CCT.T_UNSIGNED_16 is
   -- Returns number of events which yet has not been read from HSL_BUFFER.
   begin
      if (EVENT_SIZE = 0) or (HSL_BUFFER_INDEX >= HSL_BUFFER_WORDS) then
         return 0;
      end if;
      return (HSL_BUFFER_WORDS - HSL_BUFFER_INDEX - 1) / HSL_EVENT_SIZE;
      -- this -1 is because we need to have access to the marker ending
      -- the last event, which logically belongs to the next event.
   end HSL_EVENTS_LEFT;


   procedure CONFIGURE_HSL(WTI : in CIT.T_WORD_TRANSFER_INTERVAL) is
   -- Called by the TASK_HSL once at the start, or on request by
   -- TC(5,3),TID=1,FID=253,Param#1=indeks(0..15) of word transf.intrvl,
   -- Param#2=max block size.
   begin
      DPE_CSSW.RESET_SERIAL_LINE_MODULE;
      DPE_CSSW.SELECT_HIGH_SPEED_LINE (HSL_CHANNEL);
      DPE_CSSW.CONFIGURE_HIGH_SPEED_LINE (WTI);
   end CONFIGURE_HSL;


   procedure CONNECT_TO_HSL is
   -- Called by the TASK_HSL once at the start.
   begin
      DPE_CSSW.CONNECT_TO_HW_EVENT(
         HW_EVENT   => CIT.HSL_END_OF_MESSAGE,
         TASK_EVENT => HSL_RECEP_SIGNAL );

      DPE_CSSW.CONNECT_TO_HW_EVENT(
         HW_EVENT   => CIT.HSL_FIFO_FULL_ERROR,
         TASK_EVENT => HSL_OVERFLOW_ERROR_EVENT );
   end CONNECT_TO_HSL;


   procedure SET_HSL_PARAMS (
   -- Called in a response to the test TC(5,3)/TID=1/FID=253.  Was used
   -- only in the initial phase of the program development.
   -- When there were problems with HSL in Oct'2000 it was possible
   -- to bring HSL to normal operation with use of this command (iasw 1.45).
   --
      WT_INTERVAL  : in CCT.T_UNSIGNED_16;
      MAX_BLK_SIZE : in CCT.T_UNSIGNED_16
   ) is
      subtype T_WTI is CCT.T_UNSIGNED_16 range 0..15;
      WTI : array(T_WTI) of CIT.T_WORD_TRANSFER_INTERVAL := (
         CIT.CLK_000, CIT.CLK_032, CIT.CLK_064, CIT.CLK_096,
         CIT.CLK_128, CIT.CLK_160, CIT.CLK_192, CIT.CLK_224,
         CIT.CLK_256, CIT.CLK_288, CIT.CLK_320, CIT.CLK_352,
         CIT.CLK_384, CIT.CLK_416, CIT.CLK_448, CIT.CLK_480
      );
   begin
      if WT_INTERVAL not in T_WTI then
         return;
      end if;

      CONFIGURE_HSL( WTI(WT_INTERVAL) );

      if MAX_BLK_SIZE < 1 then
         HSL_MAX_READ_SIZE := 1;
      elsif MAX_BLK_SIZE > HSL_ABS_MAX_READ_SIZE then
         HSL_MAX_READ_SIZE := HSL_ABS_MAX_READ_SIZE;
      else
         HSL_MAX_READ_SIZE := MAX_BLK_SIZE;
      end if;
   end SET_HSL_PARAMS;


   function READ_HSL return BOOLEAN is
   --
   -- Reads no more than 4KW into HSL_BUFFER and performs checking for errors.  \
   -- When (CRC or) overflow error occurs, throws EX_HSL_ERROR exception.
   -- Returns TRUE when there are enough data to process.
   --
      -- HW events accepted by the procedure.
      HSL_RECEPTION : CIT.T_EVENT_LIST := (others  => FALSE);

      -- The amount of 16-bit words to be read from DFEE.
      WORDS_TO_READ : T_HSL_BUF_INDEX;

      -- HW events reported.
      SIGNALLED_EVENTS : CIT.T_EVENT_LIST;

      function CVT is new UNCHECKED_CONVERSION(
         SOURCE => CIT.T_EVENT_LIST, TARGET => CCT.T_UNSIGNED_16);
   begin
      -- Check if there is enough data in DPEE's output FIFO.
      begin
         if IASW_MRTU.GET_ANALOG_INPUT(HALF_FIFO_ADC_CHANNEL) < 75 then
            -- not enough data in FIFO
            return FALSE;
         end if;
      exception
         when others =>  -- a problem with reading ADC
            return FALSE;
      end;

      -- If there are some words left in HSL_BUFFER, move them to the
      -- beginning of the buffer.
      if HSL_BUFFER_INDEX < HSL_BUFFER_WORDS then
         HSL_BUFFER_WORDS := HSL_BUFFER_WORDS - HSL_BUFFER_INDEX;
         for i in 0 .. HSL_BUFFER_WORDS - 1 loop
            HSL_BUFFER(i) := HSL_BUFFER(HSL_BUFFER_INDEX + i);
         end loop;
      else
         HSL_BUFFER_WORDS := 0;
      end if;

      HSL_BUFFER_INDEX := 0;

      -- Calculate how many data can still fit into HSL_BUFFER.
      WORDS_TO_READ := HSL_BUFFER_SIZE - HSL_BUFFER_WORDS;
      if WORDS_TO_READ < 1 then
         -- this is a strange case where READ_HSL was called when
         -- HSL_BUFFER was full.
         return TRUE;
      end if;
      if WORDS_TO_READ > HSL_MAX_READ_SIZE then
         WORDS_TO_READ := HSL_MAX_READ_SIZE;
      end if;

      -- Try to read data from HSL and wait for response.
      HSL_RECEPTION(HSL_RECEP_SIGNAL)         := TRUE;
      HSL_RECEPTION(HSL_OVERFLOW_ERROR_EVENT) := TRUE;

      DPE_CSSW.CLEAR_EVENTS( HSL_RECEPTION );   -- added just for case

      IASW_COMMON.INC(DIAGNOSTIC_DATA.HSL_READOUTS);
      HSL_DIAG_CODE := 0;

      begin

         -- initiate HSL transfer
         DPE_CSSW.RECEIVE_HSL_MESSAGE(
            MESSAGE_ADD => (
               STATE   => HSL_BUFFER_AS,
               LOGICAL => HSL_BUFFER(HSL_BUFFER_WORDS)'Address),
            WORD_LENGTH => SYSTEM.UNSIGNED(WORDS_TO_READ));

         DPE_CSSW.WAIT_AND_CLEAR_EVENTS(
            EVENTS   => HSL_RECEPTION,
            TIMEOUT  => HSL_TIMEOUT,
            OCCURRED => SIGNALLED_EVENTS);

         HSL_DIAG_CODE := CVT(SIGNALLED_EVENTS);

      exception
         when EX_HIGH_SPEED_LINE_IS_BUSY =>
            IASW_TM.PUT_OEM(IASW_TM.EV_HSL_BUSY);
            HSL_DIAG_CODE := 16#FFF1#;
         when others =>
            HSL_DIAG_CODE := 16#FFF9#;
      end;

      -- Analyse what happened.
      if SIGNALLED_EVENTS(HSL_RECEP_SIGNAL) then
         HSL_BUFFER_WORDS := HSL_BUFFER_WORDS + WORDS_TO_READ;
         EVENTS_TIME := 0;
         IASW_TM.HSL_ERROR_CONDITION_OFF;
         return TRUE;

      elsif SIGNALLED_EVENTS(HSL_OVERFLOW_ERROR_EVENT) then
         IASW_COMMON.INC(DIAGNOSTIC_DATA.OVERFLOWS);
         IASW_TM.PUT_OEM (IASW_TM.EV_HSL_OVERFLOW);
         raise EX_HSL_ERROR;

      else  -- Timeout
         IASW_COMMON.INC(DIAGNOSTIC_DATA.TIMEOUTS);
         IASW_TM.PUT_OEM (IASW_TM.EV_HSL_TIMEOUT);
         raise EX_HSL_ERROR;

      end if;
   end READ_HSL;


   function NEXT_EVENT_SYNCHRONIZED(
   --
   -- Returns TRUE when the event-record stored in HSL_BUFFER under specified
   -- INDEX looks reasonable.
   -- Used in the normal state, when the previous event was o.k.
   --
      INDEX   : in T_HSL_BUF_INDEX
   ) return BOOLEAN is
   begin
pragma Suppress(all_checks);

      return (HSL_BUFFER(INDEX) = EVENT_MARKER) and
             (HSL_BUFFER(INDEX+HSL_EVENT_SIZE) = EVENT_MARKER);
   end NEXT_EVENT_SYNCHRONIZED;


   function EVENTS_SYNCHRONIZED(
   --
   -- Checks if successive events stored in the buffer look reasonable.
   -- Used in the state of recovering of synchronization.
   --
      HK_CC  : in CCT.T_UNSIGNED_16;
      EVENTS : in CCT.T_UNSIGNED_16;
      INDEX  : in T_HSL_BUF_INDEX
   ) return BOOLEAN is
      Bhk : CIT.T_U32_BITS;
      Lhk : LONG_INTEGER;
      Bev : CIT.T_U32_BITS;
      Lev : LONG_INTEGER;
      IDX      : T_HSL_BUF_INDEX;       -- Index to data stored in the buffer.
      EVENT_CC : CCT.T_UNSIGNED_16;
      EVENT_TC : CCT.T_UNSIGNED_16;
      PREV_CC  : CCT.T_UNSIGNED_16;
      PREV_TC  : CCT.T_UNSIGNED_16;

      function CVT is new UNCHECKED_CONVERSION(
         SOURCE => CIT.T_U32_BITS, TARGET => LONG_INTEGER);
   begin
pragma Suppress(all_checks);

      IDX := INDEX;

      for I in 1 .. EVENTS loop
         if HSL_BUFFER(IDX) /= EVENT_MARKER then
            return FALSE;
         end if;

         EVENT_CC := HSL_BUFFER(IDX+1);
         EVENT_TC := HSL_BUFFER(IDX+2);

         -- Event's CC should not differ more than -200/+2 from the one
         -- given in the last housekeeping block.

         -- convert numbers to long_integer.
         Bhk := (0, HK_CC);
         Bev := (0, EVENT_CC);
         -- consider effect of scrolling over FFFF
         if HK_CC > EVENT_CC then
           if (HK_CC - EVENT_CC) > 16#8000# then
              Bev.HIGH := 1;  -- equiv. to  Lev += 16#10000#;
           end if;
         else
           if (EVENT_CC - HK_CC) > 16#8000# then
              Bhk.HIGH := 1;  -- equiv. to  Lhk += 16#10000#;
           end if;
         end if;
         -- compare
         Lev := CVT(Bev);
         Lhk := CVT(Bhk);
         if (Lev > (Lhk+2)) or ((Lev+200) < Lhk) then
            return FALSE;
         end if;

         if I > 1 then
            -- now it is possible to compare event with the previous one.
            if EVENT_CC = PREV_CC then
               if EVENT_TC < PREV_TC then
                  return FALSE;
               end if;
            elsif EVENT_CC /= IASW_COMMON.INC_MODULO_FFFF (PREV_CC) then
               return FALSE;
            end if;
         end if;

         -- store data needed in the next iteration.
         PREV_CC  := EVENT_CC;
         PREV_TC  := EVENT_TC;
         IDX      := IDX + HSL_EVENT_SIZE;
      end loop;

      return TRUE;
   end EVENTS_SYNCHRONIZED;


   procedure GET_EVENT(
   --
   -- Retrieves next event from HSL_BUFFER.  When it is empty, executes
   -- READ_HSL.  Only first EVENT_SIZE words of EVENT are filled.  Sets
   -- RESULT to TRUE if successful.
   --
      EVENT  : in out IASW_COMMON.T_EVENT;
      RESULT : out BOOLEAN
   ) is
      REQ_EVENTS : CCT.T_UNSIGNED_16; -- number of events to be analised
      LOCAL_RESULT : BOOLEAN;
      -- LOCAL_RESULT needed to keep RESULT 'out', not 'in out'.
   begin
pragma Suppress(all_checks);

      if EVENT_SIZE = 0 then
         raise EX_EVENT_SIZE_NOT_DEFINED;
      end if;

      RESULT := FALSE;    -- preliminary
      LOCAL_RESULT := FALSE;

      -- repeat until HSL_BUFFER_INDEX points to the correct event.
      loop
         -- how many events in the buffer I need for analisis
         if SYNCHR_REQUIRED then
            REQ_EVENTS := EVENTS_TO_CHECK;
         else
            REQ_EVENTS := 1;
         end if;

         -- try to read HSL if there is not enough events in the buffer
         if HSL_EVENTS_LEFT < REQ_EVENTS then
            if not READ_HSL then
               return;   -- RESULT = FALSE;
            end if;
         end if;

         -- its ok when test mode - no synchronization is needed
         if not SYNCHR_USED then
            LOCAL_RESULT := TRUE;
            RESULT := TRUE;
            exit;
         end if;

         if not SYNCHR_REQUIRED then
         -- so far it was ok, check if the current event is correct

            if NEXT_EVENT_SYNCHRONIZED(
                  INDEX   => HSL_BUFFER_INDEX )
                  --PREV_CC => LAST_EVENT_CC,
                  --PREV_TC => LAST_EVENT_TC)
            then
               LOCAL_RESULT := TRUE;
               RESULT := TRUE;
               exit;
            else
               -- the synchronization has just been lost.
               SET_LSD;
               if AUTOMATIC_LSD then
                  SEND_LSD;
               end if;
               --
               IASW_COMMON.INC(DIAGNOSTIC_DATA.LOST_SYNCHRONIZATION);
               IASW_TM.PUT_OEM (IASW_TM.EV_HSL_LOST_SYNCHRO);
               SYNCHR_REQUIRED := TRUE;
               return;   -- RESULT = FALSE;
               -- this return is needed because in the synchro-recovering
               -- mode another value of REQ_EVENTS is necessary, so it is
               -- better that GET_EVENT will execute from the beginning,
               -- but this time in the mode SYNCHR_REQUIRED.
            end if;
         end if;

         if SYNCHR_REQUIRED then
         -- error was detected, enter synchronisation procedure

            -- repeat scaning the buffer until finding of 3 consecutive events
            -- which fulfill the test EVENTS_SYNCHRONIZED.
            loop
               -- move the starting point of analisis 1 word ahead
               HSL_BUFFER_INDEX := HSL_BUFFER_INDEX + 1;
               if HSL_EVENTS_LEFT < EVENTS_TO_CHECK then
                  return;  -- RESULT = FALSE;
               end if;
               -- check if it is ok. now
               if EVENTS_SYNCHRONIZED(
                  HK_CC  => HK_CC,
                  EVENTS => EVENTS_TO_CHECK,
                  INDEX  => HSL_BUFFER_INDEX)
               then
                  SYNCHR_REQUIRED := FALSE;  -- return to simplified checking
                  LOCAL_RESULT := TRUE;
                  RESULT := TRUE;
                  exit;
               end if;
            end loop;

         end if;
         exit when LOCAL_RESULT;
      end loop;

      -- now HSL_BUFFER_INDEX points to the correct event.
      if SYNCHR_USED then
         -- copy only the event itself, without envelope.
         for i in 0 .. EVENT_SIZE - 1 loop
            EVENT(i) := HSL_BUFFER(HSL_BUFFER_INDEX + i + 1);
         end loop;
      else
         -- copy whole event record with envelope.
         for i in 0 .. EVENT_SIZE - 1 loop
            EVENT(i) := HSL_BUFFER(HSL_BUFFER_INDEX + i);
         end loop;
      end if;

      HSL_BUFFER_INDEX := HSL_BUFFER_INDEX + HSL_EVENT_SIZE;

      IASW_COMMON.INC32( TOTAL_EVENTS_COUNT );
      IASW_COMMON.INC( EVENTS_RATE_CNT );
   end GET_EVENT;


   procedure LOCAL_STOP_DATA_TAKING is   -- local
   -- this procedure is called by TASK_HSL itself when it wants to
   -- stop data taking
   begin
      HSL_ACTIVE := FALSE;
      EVENT_SIZE := 0;
      HSL_EVENT_SIZE := 0;
      SYNCHR_USED := FALSE;
      SYNCHR_REQUIRED := FALSE;
      HK_CC := 0;
      EVENTS_TIME     := 0;
      EVENTS_RATE     := 0;
      EVENTS_RATE_CNT := 0;
      SCF_TAB(1) := 0;
      SCF_TAB(2) := 0;
      SCF_SELECTED := 1;
      PRIMARY_MIN_GF := DEFAULT_PRIMARY_MIN_GF;
      SECONDARY_MAX_GF := DEFAULT_SECONDARY_MAX_GF;
      CLEAR_HSL_BUFFER;
   end LOCAL_STOP_DATA_TAKING;


   procedure LOCAL_START_DATA_TAKING is   -- local
   -- this procedure is called by TASK_HSL itself, in the moment, when
   -- start is required and allowed.
   begin
      if START_SIGNAL.SYNC then
         EVENT_SIZE := START_SIGNAL.EV_SIZE;
         HSL_EVENT_SIZE := EVENT_SIZE + 1;
      else
         EVENT_SIZE := START_SIGNAL.EV_SIZE + 1;
         HSL_EVENT_SIZE := EVENT_SIZE;
      end if;
      SCF_TAB(1) := START_SIGNAL.SCF1;
      SCF_TAB(2) := START_SIGNAL.SCF2;

      GF_ANALYSIS_REQUIRED := EVENT_SIZE < 10;
      -- this means "Normal Data Taking"

      if START_SIGNAL.GFL1 = 16#FFFF# then
         PRIMARY_MIN_GF := DEFAULT_PRIMARY_MIN_GF;
      else
         PRIMARY_MIN_GF := START_SIGNAL.GFL1;
      end if;

      if START_SIGNAL.GFL2 = 16#FFFF# then
         SECONDARY_MAX_GF := DEFAULT_SECONDARY_MAX_GF;
      else
         SECONDARY_MAX_GF := START_SIGNAL.GFL2;
      end if;

      SCF_SELECTED := 1;
      SYNCHR_USED     := START_SIGNAL.SYNC;
      SYNCHR_REQUIRED := FALSE;  --it was SYNCHR_USED;
      CLEAR_HSL_BUFFER;
      EVENTS_TIME     := 0;
      EVENTS_RATE     := 0;
      EVENTS_RATE_CNT := 0;
      GREY_FILTER_REPORTED :=IASW_GF.GREY_FILTER_100;
      IASW_BUFFER.CLEAR_BUFFER;
      IASW_BUFFER.SET_EVENT_SIZE(EVENT_SIZE);
      IASW_BUFFER.ENABLE_INPUT;
      IASW_GF.CALCULATE_BUFFER_THRESHOLDS;
      IASW_GF.INIT_GF;
      SWITCH_SCF;
      BCP1_FLAG := FALSE;
      HSL_ACTIVE := TRUE;
   end LOCAL_START_DATA_TAKING;


   procedure TASK_HSL is
   --
   -- Reads events from HSL and puts them to the cyclic buffer.  Checks if
   -- there were exceptions raised and performs adequate handling.  This
   -- procedure is never ended (unless the whole device is switched off).
   --
      EVENT    : IASW_COMMON.T_EVENT;   -- Event currently processed.
      EVENT_OK : BOOLEAN;               -- Is there an event on input?
      END_OF_DATA : BOOLEAN;
      STOP : BOOLEAN;
      DUMMY_EVENT_TYPE : IASW_COMMON.T_DUMMY_EVENT;
   begin
      CONFIGURE_HSL(HSL_WORD_INTERVAL);
      CONNECT_TO_HSL;
      IASW_MRTU.CONNECT_TO_MRTU;  --because of HALF_FIFO_ADC_CHANNEL
      IASW_DFEE.CONNECT_TO_LSL;  -- because of SET_GREY_FILTER()

      loop
         begin
            IASW_COMMON.I_AM_ALIVE(IASW_COMMON.TASK_HSL_NO);

            if STOP_SIGNAL.ON then
               LOCAL_STOP_DATA_TAKING;
               STOP_SIGNAL.ON := FALSE;
            end if;

            if START_SIGNAL.ON then
               LOCAL_START_DATA_TAKING;
               START_SIGNAL.ON := FALSE;
            end if;

            STOP := TRUE;          --preliminary

            if HSL_ACTIVE then

               if BCP1_FLAG then
                  BCP1_FLAG := FALSE;
                  if GF_ANALYSIS_REQUIRED then
                     IASW_GF.ANALYSE_GREY_FILTER;
                     FORMAT_SWITCHING_ANALISIS;
                  end if;
               end if;

               if not IASW_BUFFER.IS_BUFFER_FULL then

                  GET_EVENT (EVENT => EVENT, RESULT => EVENT_OK);

                  END_OF_DATA := FALSE;    --preliminary

                  if EVENT_OK then
                     if SYNCHR_USED then
                        DUMMY_EVENT_TYPE :=
                           IASW_COMMON.GET_DUMMY_EVENT_TYPE (EVENT);
                        case DUMMY_EVENT_TYPE is
                     when IASW_COMMON.END_OF_DATA_MARKER =>
                           END_OF_DATA := TRUE;
                     when IASW_COMMON.GREY_FILTER_REPORT =>
                           if EVENT(4) in IASW_GF.T_GREY_FILTER_INDICATOR then
                              GREY_FILTER_REPORTED := EVENT(4);
                           else
                              GREY_FILTER_REPORTED := IASW_GF.GREY_FILTER_100;
                           end if;
                     when others =>
                           null;
                        end case;
                     end if;
                     -- if not SYNCHR_USED then this is test mode in which
                     -- input data are not interpreted at all.
                  elsif IASW_DFEE.GET_STATE > IASW_DFEE.K_STATE_SETUP then
                     -- in Data Taking there is no checking for timeout
                     EVENTS_TIME := 0;
                  else
                     END_OF_DATA := (EVENTS_TIME > EVENTS_TIMEOUT);
                  end if;

                  if END_OF_DATA then
                     -- put dummy event record End of Data into the buffer
                     EVENT(2) := 16#FFFF#;
                     EVENT(3) := 1;
                     EVENT(4) := 0;
                     IASW_BUFFER.PUT_EVENT (EVENT);
                     -- stop TASK_HSL activity
                     LOCAL_STOP_DATA_TAKING;
                     -- update state in the DFEE context
                     if IASW_DFEE.GET_STATE > IASW_DFEE.K_STATE_SETUP then
                        IASW_DFEE.ACCEPT_STATE(IASW_DFEE.K_STATE_SETUP);
                     end if;
                  elsif EVENT_OK and not IASW_BUFFER.IS_BUFFER_LOCKED then
                     IASW_BUFFER.PUT_EVENT (EVENT);
                     STOP := FALSE;
                     -- there are data to be transferred, so the process will
                     -- go with full speed.
                  else
                     -- if the buffer is locked, events are read and lost, until
                     -- "End Of Data" is received.
                     null;
                  end if;
               end if;
            end if;

            if STOP then
               DPE_CSSW.DELAY_RELATIVE (1);
            end if;

         exception
            when EX_HSL_ERROR =>
               null;

            when IASW_BUFFER.EX_INDEX_OUT_OF_RANGE |
                 IASW_BUFFER.EX_BUFFER_FULL |
                 IASW_BUFFER.EX_BUFFER_LOCKED =>
               IASW_TM.PUT_OEM (IASW_TM.EV_IASW_BUFFER, DPE_CSSW.CURRENT_TASK);

            when others =>
               IASW_TM.PUT_OEM (IASW_TM.EV_IASW_EXCEPTION, DPE_CSSW.CURRENT_TASK);
         end;
      end loop;
   end TASK_HSL;


   procedure SET_SW_PARAM(
      PARAM_ID:  in IASW_COMMON.T_FID;
      PARAM_VAL: in IASW_COMMON.T_TC_PARAM_VAL;
      PARAM_LEN: in IASW_COMMON.T_SW_PARAM_LEN
   ) is
   begin
      if PARAM_VAL'length < PARAM_LEN then
         raise IASW_COMMON.EX_SW_PARAM_ERROR_LENGTH;
      end if;

      case PARAM_ID is

   when 1 =>  -- set mode of generation of "lost synchro. dump"
         if PARAM_LEN /= 1 then
            raise IASW_COMMON.EX_SW_PARAM_ERROR_LENGTH;
         end if;
         AUTOMATIC_LSD := PARAM_VAL(1) /= 0;

   when others =>
         raise IASW_COMMON.EX_SW_PARAM_ERROR_ID;
      end case;
   end SET_SW_PARAM;

   procedure GET_SW_PARAM(
      PARAM_ID:  in  IASW_COMMON.T_FID;
      PARAM_VAL: out IASW_COMMON.T_TM_PARAM_VAL;
      PARAM_LEN: out IASW_COMMON.T_SW_PARAM_LEN
   ) is
      LSDL : constant IASW_COMMON.T_SW_PARAM_LEN := T_LOST_SYNCHRO_DUMP'size / 16;
      LSD1 : T_LOST_SYNCHRO_DUMP;
         for LSD1 use at PARAM_VAL'address;
pragma If(((LSD1'size+15)/16) > PARAM_VAL'length);
   error: Violated safety of 'use at'
pragma End if;
   begin
      case PARAM_ID is

   when 1 =>  -- get "lost synchro. dump"
         PARAM_LEN := LSDL;
         if PARAM_VAL'length < LSDL then
            raise IASW_COMMON.EX_SW_PARAM_ERROR_LENGTH;
         end if;
         LSD1 := LSD;

   when others =>
         PARAM_LEN := 0;
         raise IASW_COMMON.EX_SW_PARAM_ERROR_ID;
      end case;
   end GET_SW_PARAM;


   ------------------------------------------------------------------------
   -- GET_DPE_STATE
   --
   -- Used in TASK_MAIN to collect HK TM.

   procedure GET_DPE_STATE(DS : out IASW_COMMON.T_DPE_STATE) is
   begin
      DS.EVENT_SIZE             := EVENT_SIZE;
      DS.HSL_BUFFER_SIZE        := HSL_BUFFER_WORDS;
      DS.HSL_BUFFER_INDEX       := HSL_BUFFER_INDEX;
      DS.TIME_SINCE_READOUT     := EVENTS_TIME;
      DS.EVENTS_TRANSFERRED     := EVENTS_RATE;
      DS.DPE_EVENTS             := TOTAL_EVENTS_COUNT;
      DS.SCF(1) := CCT.T_UNSIGNED_8(SCF_TAB(1));
      DS.SCF(2) := CCT.T_UNSIGNED_8(SCF_TAB(2));
      if not HSL_ACTIVE then
         DS.ACTIVE_SCF := 0;
      else
         DS.ACTIVE_SCF := SCF_SELECTED;
      end if;
      DS.GF(2) := CCT.T_UNSIGNED_8(GREY_FILTER_REPORTED);
   end GET_DPE_STATE;


   procedure GET_HK_DIAG( FLAGS : out IASW_COMMON.T_16_BITS ) is
   begin
      FLAGS(6) := SYNCHR_REQUIRED;
      FLAGS(7) := HSL_ACTIVE;
   end GET_HK_DIAG;


   procedure BCP1_TICK is
   begin
      BCP1_FLAG := TRUE;
      -- input data throughput
      EVENTS_RATE     := EVENTS_RATE_CNT;
      EVENTS_RATE_CNT := 0;
      -- counter of time of silence on HSL
      IASW_COMMON.INC(EVENTS_TIME);
   end BCP1_TICK;


   procedure NEW_HK_CC(CC : in CCT.T_UNSIGNED_16) is
   -- called after successfull readout of DFEE HK.
   begin
      HK_CC := CC;
   end NEW_HK_CC;


   function GET_EVENTS_RATE return T_EVENTS_RATE is
   begin
      return EVENTS_RATE;
   end GET_EVENTS_RATE;


   procedure START_DATA_TAKING(   -- public
   -- this procedure is called in IASW_STATE.START_DATA_TAKING();
   -- by an external task (TC or STATE).
      EV_SIZE   : in IASW_COMMON.T_EVENT_SIZE;
      SCF1,SCF2 : in IASW_MODES.T_SCIENCE_FORMAT;
      GFL1,GFL2 : in CCT.T_UNSIGNED_16;
      SYNC      : in BOOLEAN
   ) is
   begin
      -- wait until TASK_HSL consumes the previous signal
      loop
         exit when not START_SIGNAL.ON;
         DPE_CSSW.DELAY_RELATIVE(1);
      end loop;

      if GFL1 /= 16#FFFF# then
         if GFL1 not in IASW_GF.T_GREY_FILTER_INDICATOR then
            raise IASW_MAIN.EX_DATA;
         end if;
      end if;
      if GFL2 /= 16#FFFF# then
         if GFL2 not in IASW_GF.T_GREY_FILTER_INDICATOR then
            raise IASW_MAIN.EX_DATA;
         end if;
      end if;

      START_SIGNAL.EV_SIZE := EV_SIZE;
      START_SIGNAL.SCF1    := SCF1;
      START_SIGNAL.SCF2    := SCF2;
      START_SIGNAL.GFL1    := GFL1;
      START_SIGNAL.GFL2    := GFL2;
      START_SIGNAL.SYNC    := SYNC;

      START_SIGNAL.ON      := TRUE;
   end START_DATA_TAKING;


   procedure HARD_STOP is   -- public
   begin
      STOP_SIGNAL.ON := TRUE;
   end HARD_STOP;


   function STOPPED return BOOLEAN is   -- public
   -- this is tested in IASW_MODES.STOP_DATA_TAKING(IMM => TRUE);
   begin
      return not HSL_ACTIVE;
   end STOPPED;


   procedure CLEAR_DIAGNOSTIC_DATA is
   begin
      DIAGNOSTIC_DATA.HSL_READOUTS         := 0;
      DIAGNOSTIC_DATA.CRC_ERRORS           := 0;
      DIAGNOSTIC_DATA.TIMEOUTS             := 0;
      DIAGNOSTIC_DATA.OVERFLOWS            := 0;
      DIAGNOSTIC_DATA.LOST_SYNCHRONIZATION := 0;
   end CLEAR_DIAGNOSTIC_DATA;


   procedure GET_AND_CLEAR_DIAGNOSTIC_DATA (DATA : out T_DIAGNOSTIC_DATA) is
   begin
      DATA := DIAGNOSTIC_DATA;
      CLEAR_DIAGNOSTIC_DATA;
   end GET_AND_CLEAR_DIAGNOSTIC_DATA;


   procedure CONSTRUCTOR is
   begin
      -- Set up package variables.
      LOCAL_STOP_DATA_TAKING;
      BCP1_FLAG := FALSE;
      TOTAL_EVENTS_COUNT := (0,0);
      CLEAR_DIAGNOSTIC_DATA;
      GREY_FILTER_REPORTED := 0;
      --
      AUTOMATIC_LSD := FALSE;
      CLEAR_LSD;
      --
      START_SIGNAL.ON := FALSE;
      STOP_SIGNAL.ON := FALSE;
   end CONSTRUCTOR;

end IASW_HSL;
