----------------------------------------------------------------------------
-- Module Name:        IASW_MRTU - package body
-- Project:            INTEGRAL, JEM-X, DPE
-- Author:             Dariusz Jaszkowski
-- Creation Date:      16.III.1998
-- Change History:
--    (Version/Date/Author/Description)
-- Function:
--    (Say what the component does)
--    Provides access to the A/D converter
-- Interfaces:
--    (Define the inputs and outputs)
-- Dependencies:
--    (Describe the preconditions for using this component)
--    Each task accessing A/D converter must first call CONNECT_TO_MRTU once.
-- Processing:
--    (Summarise the processing using pseudo-code or PDL)
--    Procedure GET_ANALOG_INPUT returns the analog input on a given
--    channel, converted to the binary range 0..255.  When the converter
--    is busy, procedure tries again, but no more than 10 times.  Afterthat
--    exception is raised.  In case of other type errors exception is raised
--    immediatelly.
----------------------------------------------------------------------------

with CSSW_COMMON_TYPES; use CSSW_COMMON_TYPES;
with CSSW_IF_TYPES;     use CSSW_IF_TYPES;
with IASW_COMMON;
with DPE_CSSW;
with IASW_TM;

package body IASW_MRTU is

   procedure CONNECT_TO_MRTU is
   -- this should be used at startup of each of the tasks reading ADC.
   begin
      DPE_CSSW.CONNECT_TO_HW_EVENT(
         CIT.END_OF_ANALOG_ACQUISITION,
         IASW_COMMON.MRTU_ACQUISITION_COMPLETED_EVENT);
   end CONNECT_TO_MRTU;


   function GET_ANALOG_INPUT(
      CHANNEL_ID : in CIT.T_ANALOG_CHANNEL
   ) return CIT.T_ANALOG_VALUE is
      -- task event list:
      EVENT_LIST : CIT.T_EVENT_LIST := (others => false);
      -- acquired event list:
      ACQUIRED_EVENTS : CIT.T_EVENT_LIST;
      RETRY : CCT.T_UNSIGNED_16;
      ANALOG_VALUE : CIT.T_ANALOG_VALUE;
   begin
      EVENT_LIST(IASW_COMMON.MRTU_ACQUISITION_COMPLETED_EVENT) := true;
      DPE_CSSW.CONFIGURE_ANALOG_ACQUISITION(FALSE);
      DPE_CSSW.CLEAR_EVENTS(EVENT_LIST);

      RETRY := MRTU_BUSY_RETRIES;
      loop
         begin
            DPE_CSSW.START_ANALOG_ACQUISITION(CHANNEL_ID);
            exit;    -- success, exit the loop
         exception
            when CIT.EX_ANALOG_ACQUISITION_IS_BUSY =>
               begin
                  -- dummy fetch of a value, only for clearing the "busy" flag
                  DPE_CSSW.READ_LAST_ANALOG_ACQUIRED_VALUE(ANALOG_VALUE);
               exception
                  -- this for sure will be raised
                  when others => null;
               end;

               RETRY := RETRY - 1;
               if RETRY > 0 then
                  IASW_COMMON.INC(MRTU_DIAGNOSTIC.ADC_BUSY_COUNT);
                  DPE_CSSW.DELAY_RELATIVE(1);
               else
                  IASW_COMMON.INC(MRTU_DIAGNOSTIC.ADC_ERROR_COUNT);
                  IASW_TM.PUT_OEM(IASW_TM.EV_MRTU_BUSY);
                  raise EX_ANALOG_INPUT_ERROR;
               end if;

            when others =>
               IASW_COMMON.INC(MRTU_DIAGNOSTIC.ADC_ERROR_COUNT);
               IASW_TM.PUT_OEM(IASW_TM.EV_MRTU_ERROR);
               raise EX_ANALOG_INPUT_ERROR;
         end;
      end loop;

      DPE_CSSW.WAIT_AND_CLEAR_EVENTS(EVENT_LIST, ANALOG_INPUT_TIMEOUT, ACQUIRED_EVENTS);
      if not ACQUIRED_EVENTS(IASW_COMMON.MRTU_ACQUISITION_COMPLETED_EVENT) then
         IASW_COMMON.INC(MRTU_DIAGNOSTIC.ADC_TIMEOUT_COUNT);
         IASW_TM.PUT_OEM(IASW_TM.EV_MRTU_TIMEOUT);
         raise EX_ANALOG_INPUT_ERROR;
      end if;

      begin
         DPE_CSSW.READ_LAST_ANALOG_ACQUIRED_VALUE(ANALOG_VALUE);
      exception
         when others =>
            IASW_COMMON.INC(MRTU_DIAGNOSTIC.ADC_ERROR_COUNT);
            IASW_TM.PUT_OEM(IASW_TM.EV_MRTU_ERROR);
            raise EX_ANALOG_INPUT_ERROR;
      end;
      IASW_COMMON.INC(MRTU_DIAGNOSTIC.ADC_READ_COUNT);
      IASW_TM.MRTU_ERROR_CONDITION_OFF;
      return ANALOG_VALUE;
   end GET_ANALOG_INPUT;


   procedure CLEAR_DIAGNOSTIC_DATA is
   begin
      MRTU_DIAGNOSTIC.ADC_READ_COUNT    := 0;
      MRTU_DIAGNOSTIC.ADC_ERROR_COUNT   := 0;
      MRTU_DIAGNOSTIC.ADC_BUSY_COUNT    := 0;
      MRTU_DIAGNOSTIC.ADC_TIMEOUT_COUNT := 0;
   end;


   procedure GET_AND_CLEAR_DIAGNOSTIC_DATA(DATA : out T_MRTU_DIAGNOSTIC) is
   begin
      DATA := MRTU_DIAGNOSTIC;
      CLEAR_DIAGNOSTIC_DATA;
   end GET_AND_CLEAR_DIAGNOSTIC_DATA;


   procedure CONSTRUCTOR is
   begin
      DPE_CSSW.RESET_MRTU_MODULE;
      CLEAR_DIAGNOSTIC_DATA;
   end CONSTRUCTOR;

end IASW_MRTU;
