----------------------------------------------------------------------------
-- Module Name:        IASW_REQ_OUT - package body
-- Project:            INTEGRAL, JEM-X, DPE
-- Author:             Dariusz Jaszkowski
-- Creation Date:      16.III.1998
-- Change History:
--    (Version/Date/Author/Description)
-- Function:
--    (Say what the component does)
--    Task REQ_OUT executes those telecommands generating on-request
--    telemetry.
-- Interfaces:
--    (Define the inputs and outputs)
-- Dependencies:
--    (Describe the preconditions for using this component)
-- Processing:
--    (Summarise the processing using pseudo-code or PDL)
--    The caller (task TC) uses procedure ADD_REQUEST with a telecommand
--    as an argument.  This procedure adds the request to the queue, and
--    next activates task REQ_OUT, which job is to peek requests from the
--    queue and to execute them one by one.  Requests for memory dump are
--    treated differently than others, both when putting to the queue and
--    when taking them from the queue.  Normally requests are put to the
--    queue in the order of occurance, requests for memory dump have lower
--    priority, so if there are requests for m.d. in the queue, other
--    request is put before them.
--    One request for m.d. can generate more than one telemetry packet.
--    When executing request for m.d. task REQ_OUT does not remove the
--    request from the queue until all TM packets are sent.  After sending
--    a TM packet the address and length of the memory dump are modified
--    inside the request.  Thanks to that it is possible to execute
--    another on-request telecommand during ongoing memory dump.
--    Task REQ_OUT terminates when sees empty queue.
----------------------------------------------------------------------------

with SYSTEM;
with CSSW_IF_TYPES;     use CSSW_IF_TYPES;
with CSSW_COMMON_TYPES; use CSSW_COMMON_TYPES;
with DPE_CSSW;
with IASW_COMMON;
with IASW_DFEE;
with IASW_SC_OUT;
with IASW_TM;
with IASW_HK;
with IASW_MAIN;
with IASW_HSL;

package body IASW_REQ_OUT is

   procedure CLEAR_REQUEST_BUFFER is
   begin
      DPE_CSSW.ENTER_CRITICAL_REGION;
      REQUEST_COUNT := 0;
      FIRST_REQUEST := 0;
      DPE_CSSW.LEAVE_CRITICAL_REGION;
   end CLEAR_REQUEST_BUFFER;


   -------------------------------------------------------------------------------
   --  Puts the request into the request buffer. The request is
   --  placed at the end of the buffer. If task_memory_dump is
   --  in NON_OPERATIONAL state, the ADD_REQUEST procedure will
   --  start (or re-commence) the task,
   --
   procedure ADD_REQUEST(REQUEST: in T_ON_RQ_TM_REQUEST) is
      I, N: INTEGER;
   begin
      -- check validity of the request
      if (REQUEST.CODE = MEMORY_DUMP) or (REQUEST.CODE = MEMORY_CRC) then
         if ((REQUEST.MEM_HDR.ADDR not in IASW_DFEE.T_DFEE_MEM_ADDRESS) or
             (REQUEST.MEM_HDR.LEN < 1) or
            ((REQUEST.MEM_HDR.ADDR + IASW_COMMON.CVT(REQUEST.MEM_HDR.LEN) - 1)
             not in IASW_DFEE.T_DFEE_MEM_ADDRESS))
         then
            raise IASW_MAIN.EX_ADDRESS;
         end if;
      end if;

      DPE_CSSW.ENTER_CRITICAL_REGION;

      if REQUESTS_QUEUE_FULL then
         DPE_CSSW.LEAVE_CRITICAL_REGION;
         raise EX_REQUEST_QUEUE_FULL;
      end if;

      if REQUEST.CODE = MEMORY_DUMP then
         -- The request for memory dump will be placed at the end of
         -- the queue - on the position I-th into the buffer.
         I := (FIRST_REQUEST+REQUEST_COUNT) mod REQUEST_QUEUE'length;
      else
         -- Other requests (not memory dump) are inserted at the end of
         -- the regular queue, but before requests for memory dump.
         -- The part of the queue containing requests differente than
         -- memory dump will be shifted down one position, to make room
         -- for the new request.  At the end 'I' will be pointing the free
         -- position for inserting the request.

         -- move downwards the index pointing start of the queue.
         FIRST_REQUEST :=
            (FIRST_REQUEST+REQUEST_QUEUE'length-1) mod REQUEST_QUEUE'length;

         -- shift down requests differente than memory dump.
         I := FIRST_REQUEST;
         for j in 1..REQUEST_COUNT loop
            N := (I+1) mod REQUEST_QUEUE'length;
            -- N points the position next to the I-th.
            exit when (REQUEST_QUEUE(N).CODE = MEMORY_DUMP);
            REQUEST_QUEUE(I) := REQUEST_QUEUE(N);
            I := N;
         end loop;
      end if;

      -- insert the request into the position I-th
      REQUEST_QUEUE(I) := REQUEST;

      -- modify the counter of requests in the queue
      REQUEST_COUNT := REQUEST_COUNT + 1;

      DPE_CSSW.LEAVE_CRITICAL_REGION;

      -- push the task executing requests to do it's job.
      IASW_COMMON.START_TASK(IASW_COMMON.TASK_REQ_OUT_ID);
   end ADD_REQUEST;


   procedure GET_REQUEST(     -- local
      REQUEST : out T_ON_RQ_TM_REQUEST;
      PARTIAL : out BOOLEAN
      -- PARTIAL will return true, if the request returned is memory dump,
      -- and the returned one does not cover the whole dump.
   ) is
   begin
      PARTIAL := FALSE;

      DPE_CSSW.ENTER_CRITICAL_REGION;

      if REQUEST_COUNT < 1 then
         DPE_CSSW.LEAVE_CRITICAL_REGION;
         raise EX_REQUEST_QUEUE_EMPTY;
      end if;

      REQUEST := REQUEST_QUEUE(FIRST_REQUEST);

      if (REQUEST_QUEUE(FIRST_REQUEST).CODE = MEMORY_DUMP) and
         (REQUEST_QUEUE(FIRST_REQUEST).MEM_HDR.LEN > IASW_DFEE.K_MAX_DUMP_MEM_BLK_LEN)
      then
         REQUEST.MEM_HDR.LEN := IASW_DFEE.K_MAX_DUMP_MEM_BLK_LEN;
         REQUEST_QUEUE(FIRST_REQUEST).MEM_HDR.ADDR:=
            REQUEST_QUEUE(FIRST_REQUEST).MEM_HDR.ADDR +
            IASW_COMMON.CVT(IASW_DFEE.K_MAX_DUMP_MEM_BLK_LEN);
         REQUEST_QUEUE(FIRST_REQUEST).MEM_HDR.LEN:=
            REQUEST_QUEUE(FIRST_REQUEST).MEM_HDR.LEN -
            IASW_DFEE.K_MAX_DUMP_MEM_BLK_LEN;
         PARTIAL := TRUE;
      else
         FIRST_REQUEST := (FIRST_REQUEST + 1) mod REQUEST_QUEUE'length;
         REQUEST_COUNT := REQUEST_COUNT - 1;
      end if;

      DPE_CSSW.LEAVE_CRITICAL_REGION;
   end GET_REQUEST;


   procedure REPORT_TASK_PARAMS(REQUEST : in T_ON_RQ_TM_REQUEST) is  -- local
      type T_TM_PACKET is record
         H : IASW_COMMON.T_ON_RQ_TM_HEADER;
         PARAM_ID  : IASW_COMMON.T_PARAM_ID;
         PARAM_VAL : IASW_COMMON.T_TM_PARAM_VAL;
      end record;

      TM_DATA : CIT.T_SCIENCE_DATA_FIELD;
      TM : T_TM_PACKET;
         for TM use at TM_DATA'address;
pragma If(((TM'size+15)/16) > TM_DATA'length);
   error: Violated safety of 'use at'
pragma End if;

      PARAM_LEN : IASW_COMMON.T_SW_PARAM_LEN;
   begin
      IASW_TM.INIT_ON_REQUEST_TM(TM_DATA, 5, 4);
      TM.PARAM_ID := REQUEST.PARAM_ID;

      case TM.PARAM_ID.TID is
         when IASW_MAIN.TID_DFEE =>
            IASW_DFEE.GET_SW_PARAM(TM.PARAM_ID.FID, TM.PARAM_VAL, PARAM_LEN);
         when IASW_MAIN.TID_MAIN =>
            IASW_MAIN.GET_SW_PARAM(TM.PARAM_ID.FID, TM.PARAM_VAL, PARAM_LEN);
         when IASW_MAIN.TID_HSL =>
            IASW_HSL.GET_SW_PARAM(TM.PARAM_ID.FID, TM.PARAM_VAL, PARAM_LEN);
         when IASW_MAIN.TID_SC_OUT =>
            IASW_SC_OUT.GET_SW_PARAM(TM.PARAM_ID.FID, TM.PARAM_VAL, PARAM_LEN);
         when others =>  -- invalid TID value
            raise IASW_MAIN.EX_TID_FID;
      end case;

      IASW_TM.PUT_TM_FOR_SURE(IASW_COMMON.APID_TM_5_4, TM_DATA, FALSE);
      DPE_CSSW.DELAY_RELATIVE(1);
   end REPORT_TASK_PARAMS;


   procedure DUMP_MEMORY(REQUEST: in T_ON_RQ_TM_REQUEST) is   -- local
      type T_TM_PACKET is record
         H : IASW_COMMON.T_ON_RQ_TM_HEADER;
         MEM_HDR : IASW_COMMON.T_TC_MEM_HDR;
         MEM_CONTENT : IASW_DFEE.T_MEM_BLOCK(1..IASW_DFEE.K_MAX_DUMP_MEM_BLK_LEN);
      end record;

      TM_DATA : CIT.T_SCIENCE_DATA_FIELD;
      TM : T_TM_PACKET;
         for TM use at TM_DATA'address;
pragma If(TM'size > TM_DATA'size);
   error: Violated safety of 'use at'
pragma End if;

      MEM_DESCR : IASW_DFEE.T_MEM_BLOCK_SPEC;
   begin
      IASW_TM.INIT_ON_REQUEST_TM(TM_DATA, 6, 2);

      MEM_DESCR.ADDR   := REQUEST.MEM_HDR.ADDR;
      MEM_DESCR.LENGTH := REQUEST.MEM_HDR.LEN;
      TM.MEM_HDR       := REQUEST.MEM_HDR;

      IASW_DFEE.DUMP_MEMORY(MEM_DESCR, TM.MEM_CONTENT);

      IASW_TM.PUT_TM_FOR_SURE(IASW_COMMON.APID_TM_6_2, TM_DATA, FALSE);
      DPE_CSSW.DELAY_RELATIVE(1);
   end DUMP_MEMORY;


   procedure CALCULATE_MEMORY_CRC(REQUEST: in T_ON_RQ_TM_REQUEST) is  -- local
      type T_TM_PACKET is record
         H : IASW_COMMON.T_ON_RQ_TM_HEADER;
         MEM_HDR : IASW_COMMON.T_TC_MEM_HDR;
         CRC : CCT.T_UNSIGNED_16;
      end record;

      TM_DATA : CIT.T_SCIENCE_DATA_FIELD;
      TM : T_TM_PACKET;
         for TM use at TM_DATA'address;
pragma If(((TM'size+15)/16) > TM_DATA'length);
   error: Violated safety of 'use at'
pragma End if;

      MEM_DESCR : IASW_DFEE.T_MEM_BLOCK_SPEC;
   begin
       IASW_TM.INIT_ON_REQUEST_TM(TM_DATA, 6, 3);

       MEM_DESCR.ADDR   := REQUEST.MEM_HDR.ADDR;
       MEM_DESCR.LENGTH := REQUEST.MEM_HDR.LEN;
       TM.MEM_HDR       := REQUEST.MEM_HDR;

       IASW_DFEE.GET_MEMORY_CRC(MEM_DESCR, TM.CRC);

       IASW_TM.PUT_TM_FOR_SURE(IASW_COMMON.APID_TM_6_3, TM_DATA, FALSE);
       DPE_CSSW.DELAY_RELATIVE(1);
   end CALCULATE_MEMORY_CRC;


   procedure DO_DIAG_CODE_REPORT(REQUEST: in T_ON_RQ_TM_REQUEST) is
      TM_DATA : CIT.T_SCIENCE_DATA_FIELD;
      TM_DONE : BOOLEAN;
   begin
      IASW_TM.INIT_SCIENCE_TM(TM_DATA);
      TM_DATA(1) := IASW_COMMON.CVT(REQUEST.DIAG_ID);
      TM_DATA(2) := IASW_COMMON.CVT(REQUEST.DIAG_CODE);
      TM_DATA(3) := -1;
      IASW_TM.PUT_TM(IASW_COMMON.APID_TM_SC_DEBUG, TM_DATA, TM_DONE);
   end DO_DIAG_CODE_REPORT;


   -------------------------------------------------------------------------------
   -- This task is activated by the ADD_REQUEST procedure, after the
   -- new request is put into an empty buffer.
   -- The task processes all requests from the queue, and
   -- terminates if there is no more request,
   --
   procedure TASK_REQ_OUT is
      REQUEST : T_ON_RQ_TM_REQUEST;
      PARTIAL : BOOLEAN;
      FAILED : BOOLEAN;
   begin
      IASW_DFEE.CONNECT_TO_LSL;

      loop
         while REQUEST_COUNT > 0 loop
            IASW_COMMON.I_AM_ALIVE(IASW_COMMON.TASK_REQ_OUT_NO);
            begin
               FAILED := TRUE;
               -- if execution is interrupted, FAILED will remain TRUE.

               GET_REQUEST(REQUEST, PARTIAL);

               case REQUEST.CODE is
                   when TASK_PARAMETER_REPORT =>
                       REPORT_TASK_PARAMS(REQUEST);

                   when MEMORY_DUMP =>
                       DUMP_MEMORY(REQUEST);

                   when MEMORY_CRC =>
                       CALCULATE_MEMORY_CRC(REQUEST);

                   when DIAG_CODE_REPORT =>
                       DO_DIAG_CODE_REPORT(REQUEST);
               end case;

               -- execution was successful
               FAILED := FALSE;
            exception
               when IASW_MAIN.EX_TYPE_SUBTYPE =>
                  IASW_TM.PUT_OEM(IASW_TM.EV_TC_REJ_TYPE_SUBTYPE,
                     REQUEST.OEMF2, REQUEST.OEMF3);
               when IASW_MAIN.EX_TID_FID |
                    IASW_COMMON.EX_SW_PARAM_ERROR_ID =>
                  IASW_TM.PUT_OEM(IASW_TM.EV_TC_REJ_TID_FID,
                     REQUEST.OEMF2, REQUEST.OEMF3);
               when IASW_MAIN.EX_MID =>
                  IASW_TM.PUT_OEM(IASW_TM.EV_TC_REJ_MID,
                     REQUEST.OEMF2, REQUEST.OEMF3);
               when IASW_MAIN.EX_ADDRESS =>
                  IASW_TM.PUT_OEM(IASW_TM.EV_TC_REJ_ADDRESS,
                     REQUEST.OEMF2, REQUEST.OEMF3);
               when IASW_MAIN.EX_LENGTH |
                    IASW_COMMON.EX_SW_PARAM_ERROR_LENGTH =>
                  IASW_TM.PUT_OEM(IASW_TM.EV_TC_REJ_LENGTH,
                     REQUEST.OEMF2, REQUEST.OEMF3);
               when IASW_MAIN.EX_DATA |
                    IASW_COMMON.EX_SW_PARAM_ERROR_VALUE |
                    IASW_DFEE.EX_DFEE_CALLER_ERROR =>
                  IASW_TM.PUT_OEM(IASW_TM.EV_TC_REJ_DATA,
                     REQUEST.OEMF2, REQUEST.OEMF3);
               when IASW_MAIN.EX_BUSY |
                    EX_REQUEST_QUEUE_FULL =>
                  IASW_TM.PUT_OEM(IASW_TM.EV_TC_REJ_BUSY,
                     REQUEST.OEMF2, REQUEST.OEMF3);
               when IASW_MAIN.EX_SAFETY =>
                  IASW_TM.PUT_OEM(IASW_TM.EV_TC_REJ_SAFETY,
                     REQUEST.OEMF2, REQUEST.OEMF3);
               when IASW_MAIN.EX_STATE =>
                  IASW_TM.PUT_OEM(IASW_TM.EV_TC_REJ_STATE,
                     REQUEST.OEMF2, REQUEST.OEMF3);
               when IASW_MAIN.EX_EXEC_FAILED |
                    IASW_DFEE.EX_LSL_DEAD |
                    IASW_DFEE.EX_DFEE_BUSY |
                    IASW_DFEE.EX_DFEE_DETECTS_ERROR |
                    IASW_DFEE.EX_DFEE_UNEXP_ANSWER =>
                  IASW_TM.PUT_OEM(IASW_TM.EV_TC_REJ_EXEC_FAILED,
                     REQUEST.OEMF2, REQUEST.OEMF3);

               when EX_REQUEST_QUEUE_EMPTY =>
                  IASW_TM.PUT_OEM(IASW_TM.EV_IASW_REQ_QUEUE,
                     REQUEST.OEMF2, REQUEST.OEMF3);

               when others =>
                  IASW_TM.PUT_OEM(IASW_TM.EV_IASW_EXCEPTION,
                     DPE_CSSW.CURRENT_TASK);
            end;

         end loop;
         DPE_CSSW.TERMINATE_TASK;
      end loop;
   end TASK_REQ_OUT;


   function REQUESTS_QUEUE_FULL return BOOLEAN is
   begin
       return (REQUEST_COUNT >= REQUEST_QUEUE'length);
   end REQUESTS_QUEUE_FULL;


   procedure REPORT_DIAG_CODE (
   -- designed for debuggung purposes.
      DIAG_CODE : in CCT.T_UNSIGNED_16;
      DIAG_ID   : in CCT.T_UNSIGNED_16 := 16#FFFF#
   ) is
      REQUEST : T_ON_RQ_TM_REQUEST;
   begin
      REQUEST.CODE      := DIAG_CODE_REPORT;
      REQUEST.DIAG_CODE := DIAG_CODE;
      REQUEST.DIAG_ID   := DIAG_ID;
      ADD_REQUEST(REQUEST);
   exception
      when others =>
         null;
   end REPORT_DIAG_CODE;


   procedure CONSTRUCTOR is
   begin
       CLEAR_REQUEST_BUFFER;
   end CONSTRUCTOR;

end IASW_REQ_OUT;
