----------------------------------------------------------------------------
-- Module Name:        IASW_TM - package body
-- Project:            INTEGRAL, JEM-X, DPE
-- Author:             Dariusz Jaszkowski
-- Creation Date:      3.III.1998
-- Change History:
--    (Version/Date/Author/Description)
--    see Release Notes - the separate document
-- Function:
--    (Say what the component does)
-- Interfaces:
--    (Define the inputs and outputs)
-- Dependencies:
--    (Describe the preconditions for using this component)
-- Processing:
--    (Summarise the processing using pseudo-code or PDL)
----------------------------------------------------------------------------

with system;
with UNCHECKED_CONVERSION;
with CSSW_COMMON_TYPES; use CSSW_COMMON_TYPES;
with CSSW_IF_TYPES;     use CSSW_IF_TYPES;
with IASW_COMMON;
with DPE_CSSW;
with IASW_TIME;         use IASW_TIME;
with IASW_MAIN;


package body IASW_TM is

   --------------------------------
   -- ON EVENT MESSAGES TELEMETRY
   --------------------------------

   -- LOGIC OF FILTERING OF OEM-S:
   -- This OEM Service package receives two kinds of signals from the
   -- rest of IASW:
   -- 1) Requests for issuing of OEM - via call of procedure PUT_OEM(),
   -- 2) Notifications about correct behaviour of subsystems - via calls
   --    of procedures xxx_ERROR_CONDITION_OFF;
   -- OEM Service package keeps a list of flags telling about current
   -- status of subsystems.  A flag is set on reception of PUT_OEM, and
   -- cleared on reception of xxx_ERROR_CONDITION_OFF.  The list has a
   -- form of array ERROR_FLAG(), where OEM's ID is index to it.
   -- Filtering of OEMs means that when there is a request PUT_OEM(), but
   -- the corresponding ERROR_FLAG() is set, the OEM will not be passed
   -- to the output.
   -- On reception of signal xxx_ERROR_CONDITION_OFF, a group of flags
   -- corresponding to the correct subsystem is cleared. If in the group
   -- there were flags set, additionally special OEM is generated -
   -- EV_ERROR_CONDITION_OFF, with FIELD2 indicating the subsystem or
   -- situation which again is correct:
   --   FIELD2        Correct subsystem
   --  - - - - - - - - - - - - - - - - - - - - -
   --    150          HSL
   --    160          LSL
   --    170          mRTU
   --    188          CRC on LSL
   --    189          CC in HK as expected
   --    200          BCP1
   --
   -- There is a number of OEMs which are not filtered at all.  This covers
   -- for example telecomand rejections, and autonomous actions reports.
   -- Those events has no error flags associated (the flags physically
   -- exist, but are not used).
   -- OEM 233 - "Recovery initiated" is also filtered, but not here.
   -- IASW_STATE takes care of not repeating it when waiting 10 minutes
   -- after low radiation counters.
   -- Logical sum of all error flags is so called master error flag, which
   -- is reported in HK.

   procedure CLEAR_OEMS_FILTER is   -- local
   begin
      for I in ERROR_FLAG'range loop
         ERROR_FLAG(I) := FALSE;
      end loop;
   end CLEAR_OEMS_FILTER;


   function MASTER_ERROR_FLAG return BOOLEAN is
   -- returns TRUE if any of the individual error flags is set.
   begin
      for I in ERROR_FLAG'range loop
         if ERROR_FLAG(I) then
            return TRUE;
         end if;
      end loop;
      return FALSE;
   end MASTER_ERROR_FLAG;


   function OEM_FILTERED_OUT(
      INDIC  : in T_OEM_INDICATOR;
      FIELD2 : in INTEGER;
      FIELD3 : in INTEGER
   ) return BOOLEAN is
   begin
      case INDIC is

   when EV_DFEE_STATE =>
         -- this event needs special treatment, because filtering
         -- depends also on value of data fields.
         if ERROR_FLAG(INDIC) and
            (LAST_OEM_DFEE_STATE.FIELD2 = FIELD2) and
            (LAST_OEM_DFEE_STATE.FIELD3 = FIELD3) 
         then
            return TRUE;
         end if;
         LAST_OEM_DFEE_STATE.FIELD2 := FIELD2;
         LAST_OEM_DFEE_STATE.FIELD3 := FIELD3;
         ERROR_FLAG(INDIC) := TRUE;

   when
      --EV_TC_REJ_TYPE_SUBTYPE |
      --EV_TC_REJ_TID_FID |
      --EV_TC_REJ_MID |
      --EV_TC_REJ_ADDRESS |
      --EV_TC_REJ_LENGTH |
      --EV_TC_REJ_DATA |
      --EV_TC_REJ_BUSY |
      --EV_TC_REJ_SAFETY |
      --EV_TC_REJ_STATE |
      --EV_TC_REJ_EXEC_FAILED |
      EV_HSL_OVERFLOW |
      EV_HSL_BUSY |
      EV_HSL_CRC |
      EV_HSL_TIMEOUT |
      --EV_HSL_LOST_SYNCHRO |
      EV_LSL_OVERRUN |
      EV_LSL_FRAMING |
      EV_LSL_PARITY |
      EV_LSL_DISABLED |
      EV_LSL_BUSY |
      EV_LSL_TIMEOUT |
      EV_MRTU_ERROR |
      EV_MRTU_BUSY |
      EV_MRTU_TIMEOUT |
      --EV_DFEE_UNEXP_ANSWER |
      --EV_DFEE_RA_1 |
      --EV_DFEE_RA_2 |
      --EV_DFEE_RA_3 |
      --EV_DFEE_RA_4 |
      --EV_DFEE_RA_5 |
      --EV_DFEE_RA_6 |
      --EV_DFEE_RA_X |
      EV_DFEE_CRC |
      --EV_DFEE_HK_CC |
      --EV_DFEE_CONTEXT |
      --EV_DFEE_HV_OFF_RELAY |
      EV_CLOCK_BCP1_SHORT |
      EV_CLOCK_BCP1_LONG --|
      --EV_IASW_EXCEPTION |
      --EV_IASW_BUFFER |
      --EV_IASW_REQ_QUEUE |
      --EV_IASW_PATCHES |
      --EV_CSSW_EXCEPTION |
      --EV_CSSW_NO_RTC |
      --EV_AUTO_SHUTDOWN |
      --EV_AUTO_SHD_FAILED |
      --EV_AUTO_RECOVERY |
      --EV_AUTO_RCV_FAILED |
      --EV_AUTO_RESTART |
      --EV_TM_LOST |
      --EV_ERROR_CONDITION_OFF |
      =>
         -- these are events of type "error condition on"
         if ERROR_FLAG(INDIC) then
            return TRUE;
         end if;
         ERROR_FLAG(INDIC) := TRUE;

   when others =>
         -- others are never filtered.
         null;
      end case;
      return FALSE;
   end OEM_FILTERED_OUT;


   procedure LSL_ERROR_CONDITION_OFF is
   -- this is called in IASW_DFEE.LSL_TX_RX()
   begin
      if ERROR_FLAG(EV_LSL_OVERRUN) or
         ERROR_FLAG(EV_LSL_FRAMING) or
         ERROR_FLAG(EV_LSL_PARITY) or
         ERROR_FLAG(EV_LSL_DISABLED) or
         ERROR_FLAG(EV_LSL_BUSY) or
         ERROR_FLAG(EV_LSL_TIMEOUT)
      then
         ERROR_FLAG(EV_LSL_OVERRUN) := FALSE;
         ERROR_FLAG(EV_LSL_FRAMING) := FALSE;
         ERROR_FLAG(EV_LSL_PARITY) := FALSE;
         ERROR_FLAG(EV_LSL_DISABLED) := FALSE;
         ERROR_FLAG(EV_LSL_BUSY) := FALSE;
         ERROR_FLAG(EV_LSL_TIMEOUT) := FALSE;
         PUT_OEM(EV_ERROR_CONDITION_OFF, 160);
      end if;
   end LSL_ERROR_CONDITION_OFF;


   procedure DFEE_CRC_ERROR_CONDITION_OFF is
   -- this is called in IASW_DFEE.DPE_DFEE_DIALOG()
   begin
      if ERROR_FLAG(EV_DFEE_CRC) then
         ERROR_FLAG(EV_DFEE_CRC) := FALSE;
         PUT_OEM(EV_ERROR_CONDITION_OFF, CVT(OEM_DESCRIPTOR(EV_DFEE_CRC).ID));
      end if;
   end DFEE_CRC_ERROR_CONDITION_OFF;


   procedure DFEE_STATE_ERROR_CONDITION_OFF is
   -- this is called in IASW_DFEE.CHECK_STATE()
   begin
      if ERROR_FLAG(EV_DFEE_STATE) then
         ERROR_FLAG(EV_DFEE_STATE) := FALSE;
         PUT_OEM(EV_ERROR_CONDITION_OFF, CVT(OEM_DESCRIPTOR(EV_DFEE_STATE).ID));
      end if;
   end DFEE_STATE_ERROR_CONDITION_OFF;


   procedure HSL_ERROR_CONDITION_OFF is
   -- this is called in IASW_HSL.READ_HSL()
   begin
      if ERROR_FLAG(EV_HSL_OVERFLOW) or
         ERROR_FLAG(EV_HSL_BUSY) or
         ERROR_FLAG(EV_HSL_CRC) or
         ERROR_FLAG(EV_HSL_TIMEOUT)
      then
         ERROR_FLAG(EV_HSL_OVERFLOW) := FALSE;
         ERROR_FLAG(EV_HSL_BUSY) := FALSE;
         ERROR_FLAG(EV_HSL_CRC) := FALSE;
         ERROR_FLAG(EV_HSL_TIMEOUT) := FALSE;
         PUT_OEM(EV_ERROR_CONDITION_OFF, 150);
      end if;
   end HSL_ERROR_CONDITION_OFF;


   procedure MRTU_ERROR_CONDITION_OFF is
   -- this is called in IASW_MRTU.GET_ANALOG_INPUT()
   begin
      if ERROR_FLAG(EV_MRTU_ERROR) or
         ERROR_FLAG(EV_MRTU_BUSY) or
         ERROR_FLAG(EV_MRTU_TIMEOUT)
      then
         ERROR_FLAG(EV_MRTU_ERROR) := FALSE;
         ERROR_FLAG(EV_MRTU_BUSY) := FALSE;
         ERROR_FLAG(EV_MRTU_TIMEOUT) := FALSE;
         PUT_OEM(EV_ERROR_CONDITION_OFF, 170);
      end if;
   end MRTU_ERROR_CONDITION_OFF;


   procedure BCP1_ERROR_CONDITION_OFF is
   -- this is called in IASW_MAIN.TASK_MAIN()
   begin
      if ERROR_FLAG(EV_CLOCK_BCP1_SHORT) or
         ERROR_FLAG(EV_CLOCK_BCP1_LONG)
      then
         ERROR_FLAG(EV_CLOCK_BCP1_SHORT) := FALSE;
         ERROR_FLAG(EV_CLOCK_BCP1_LONG) := FALSE;
         PUT_OEM(EV_ERROR_CONDITION_OFF, 200);
      end if;
   end BCP1_ERROR_CONDITION_OFF;


   procedure PUT_OEM(
      INDIC  : in T_OEM_INDICATOR;
      FIELD2 : in INTEGER := 0;
      FIELD3 : in INTEGER := 0
   ) is
      MSG_BODY : CIT.T_ON_EVENT_MSG_BODY;
      MSG_HDR :  CIT.T_IASW_ON_EVENT_MSG_ID;
   begin
      -- check if it is ok. to emit this OEM
      if OEM_FILTERED_OUT(INDIC,FIELD2,FIELD3) then
         IASW_COMMON.INC(OEMS_REJECTED,1,0);
         return;   --ignore this OEM
      end if;

      MSG_BODY := (
         FIELD_1 => OEM_DESCRIPTOR(INDIC).F1,
         FIELD_2 => FIELD2,
         FIELD_3 => FIELD3 );
      MSG_HDR := (
         ID    => OEM_DESCRIPTOR(INDIC).ID,
         CLASS => OEM_DESCRIPTOR(INDIC).CL );
      begin
         DPE_CSSW.PUT_ON_EVENT_MESSAGE(MSG_HDR, MSG_BODY);
      exception
         when others =>
            null;
      end;
   end PUT_OEM;


   ------------------------------------
   -- SCIENCE & ON-REQUEST TELEMETRY
   ------------------------------------


   procedure INIT_SCIENCE_TM(
      DATA : out CIT.T_SCIENCE_DATA_FIELD
   ) is
   begin
      for I in DATA'range loop
         DATA(I) := 0;
      end loop;
   end INIT_SCIENCE_TM;


   procedure INIT_ON_REQUEST_TM(
      DATA : out CIT.T_SCIENCE_DATA_FIELD;
      TYP  : in IASW_COMMON.T_PACKET_TYPE;
      SUB  : in IASW_COMMON.T_PACKET_SUBTYPE
   ) is
      subtype T_B6 is IASW_COMMON.T_PACKED_ARRAY_OF_BYTES(1..6);
      BOBT : T_B6;

      DESCR : IASW_COMMON.T_ON_RQ_TM_HEADER;
         for DESCR use at DATA'address;
pragma If(((DESCR'size+15)/16) > DATA'length);
   error: Violated safety of 'use at'
pragma End if;

      function CVT is new UNCHECKED_CONVERSION(
         SOURCE => CIT.T_OBT, TARGET => T_B6);
   begin
      INIT_SCIENCE_TM(DATA);

      -- put the packet type into th TM data:
      DESCR.TYP := TYP;
      DESCR.SUB := SUB;

      -- put a timestamp [sec] into the TM data:
      BOBT := CVT(DPE_CSSW.GET_CURRENT_OBT);
      DESCR.TIME1 := BOBT(2);
      DESCR.TIME2 := BOBT(3);
   end INIT_ON_REQUEST_TM;


   procedure PUT_TM(
      APID : in CIT.T_IASW_APID;
      DATA : in CIT.T_SCIENCE_DATA_FIELD;
      DONE : out BOOLEAN;
      SCIENCE : in BOOLEAN := TRUE
   ) is
      PRIORITY : CIT.T_SCI_TM_PRIORITY;

      DESCR : IASW_COMMON.T_ON_RQ_TM_HEADER;
         for DESCR use at DATA'address;
pragma If(((DESCR'size+15)/16) > DATA'length);
   error: Violated safety of 'use at'
pragma End if;

   begin
      DONE := FALSE;

      if APID not in T_TM_APID_RANGE then
         raise CIT.EX_APID_NOT_IN_USE;
      end if;

      --added in ver.1.62
      --to minimize exceptions 'ICB full'.
      if DPE_CSSW.GET_ICB_STATE(CIT.TM_SCI).FREE < 1 then
         return;   -- with DONE = FALSE
      end if;

      if SCIENCE then
         PRIORITY := CIT.LOW;
      else
         PRIORITY := CIT.MEDIUM;
      end if;

      begin
         DPE_CSSW.PUT_SCIENCE_TM(APID, DATA, SEQ_COUNT(APID), PRIORITY, TRUE);
         DONE := TRUE;
         if SEQ_COUNT(APID) = 16#3FFF# then   --SEQ_COUNT is 14 bits long
            SEQ_COUNT(APID) := 0;
         else
            SEQ_COUNT(APID) := SEQ_COUNT(APID) + 1;
         end if;
      exception
         when CIT.EX_SCI_SB_FULL | CIT.EX_OE_SB_FULL =>
            null;
            -- DONE remains FALSE, and the caller may repeat PUT_TM().
         when others =>
            PUT_OEM(EV_CSSW_EXCEPTION, DPE_CSSW.CURRENT_TASK);
      end;
   end PUT_TM;


   procedure PUT_TM_FOR_SURE(
   -- this is for those callers who don't want to bother with parameter DONE
   -- in the oryginal PUT_TM().
      APID : in CIT.T_IASW_APID;
      DATA : in CIT.T_SCIENCE_DATA_FIELD;
      SCIENCE : in BOOLEAN := TRUE
   ) is
      DONE : BOOLEAN;
   begin
      loop
         PUT_TM(APID, DATA, DONE, SCIENCE);
         exit when DONE;
         -- return processor for other tasks for a while before
         -- repeating PUT_TM().
         DPE_CSSW.DELAY_RELATIVE(1);
      end loop;
   end PUT_TM_FOR_SURE;


   procedure BCP1_TICK is
   begin
      null;
   end BCP1_TICK;


   procedure GET_HK_DIAG(
      FLAGS : out IASW_COMMON.T_16_BITS;
      DIAG  : out T_DIAGNOSTIC_HK
   ) is
   begin
      FLAGS(8) := MASTER_ERROR_FLAG;
      DIAG.OEMS_REJECTED := OEMS_REJECTED;
   end GET_HK_DIAG;


   procedure CONSTRUCTOR is
   begin
      for I in T_TM_APID_RANGE loop
         SEQ_COUNT(I) := 0;
      end loop;

      CLEAR_OEMS_FILTER;
      OEMS_REJECTED := 0;
   end CONSTRUCTOR;

end IASW_TM;
